/***************************************************************************
                          dcchat.cpp  -  description
                             -------------------
    begin                : Don Mr 28 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdatetime.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qclipboard.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qimage.h>
#include <qregexp.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qprocess.h>
#include <qglobal.h>
#include <qstatusbar.h>
#include <qtooltip.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qsplitter.h>

#include <dcgui.h>
#include <dcevent.h>
#include <dcclient.h>
#include <dcmenuhandler.h>
#include <dcconfig.h>
#include <dctranslator.h>

#include "dcchat.h"

#include <dclib/cfilemanager.h>
#include <dclib/dcos.h>

#include <res/encrypted.xpm>
#include <res/decrypted.xpm>

/** the link list*/
const char * p_LinkList[] = {
	"HTTP://",
	"HTTPS://",
	"NEWS://",
	"FILE://",
	"WWW.",
	"FTP://",
	NULL
};

/** */
DCChat::DCChat( QWidget * parent, const char *name, int wflags, DCClient * client, bool bprivate ) : DCDialogChat(parent, name, wflags)
{
	ASSERT(client);

	m_bPrivateChat = bprivate;
	m_pClient      = client;
	bSendAway      = TRUE;
	m_nEncrypted   = 0;
	m_pCurrentHistory = 0;

	InitDocument();
}

/** */
DCChat::~DCChat()
{
}

/** */
void DCChat::InitDocument()
{
	int i;
	m_pStatusBar = new QStatusBar(this);
	DCDialogChatLayout->addWidget( m_pStatusBar, 1, 0 );
	m_pStatusCrypt = new QLabel(m_pStatusBar);
	m_pStatusBar->addWidget(m_pStatusCrypt);
	SetCrypt(FALSE);

	if ( !m_bPrivateChat )
	{
		m_pStatusBar->hide();
	}

/*	if ( Config->ShowChatStatusBar() == FALSE )
	{
		m_pStatusBar->hide();
	}
*/

	if ( Config->GetShowChatSendButton() )
	{
		connect( PushButton_SEND, SIGNAL(clicked()), this, SLOT(slotSendMessage()) );
	}
	else
	{
		// remove send button and recreate layout
		delete PushButton_SEND;
		delete Frame_SEND->layout();
		QGridLayout * gl = new QGridLayout( Frame_SEND, 1, 1, 1, 1, "");
		gl->addWidget( TextEdit_CHATINPUT, 0, 0 );
	}

	TextEdit_CHATOUTPUT = new DCQTextEdit( Frame_OUTPUT, "TextEdit_CHATOUTPUT" );
	TextEdit_CHATOUTPUT->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, TextEdit_CHATOUTPUT->sizePolicy().hasHeightForWidth() ) );
	TextEdit_CHATOUTPUT->setMinimumSize( QSize( 0, 60 ) );
	TextEdit_CHATOUTPUT->setTextFormat( QTextEdit::RichText );
	TextEdit_CHATOUTPUT->setWordWrap( QTextEdit::WidgetWidth );
	TextEdit_CHATOUTPUT->setWrapPolicy( QTextEdit::AtWhiteSpace );
    	TextEdit_CHATOUTPUT->setReadOnly( TRUE );
	Frame_OUTPUTLayout->addWidget(TextEdit_CHATOUTPUT,0,0);

	connect( TextEdit_CHATOUTPUT, SIGNAL(rightButtonClicked( const QPoint& )), this, SLOT(slotRightButtonClickedChatOutput( const QPoint& )) );
	connect( TextEdit_CHATOUTPUT, SIGNAL(clicked( int, int )), this, SLOT(slotClickedChatOutput( int, int )) );

	installEventFilter(this);

	TextEdit_CHATOUTPUT->installEventFilter(this);
	TextEdit_CHATINPUT->installEventFilter(this);
	TextEdit_CHATINPUT->setFocus();

	// resize splitter
	QValueList<int> list = Splitter3->sizes();
	QValueList<int>::Iterator it = list.begin();
	i = *it;
	++it;
	i += *it;
	i -= TextEdit_CHATINPUT->height();
	it = list.begin();
	*it = i;
	++it;
	*it = TextEdit_CHATINPUT->height();
	Splitter3->setSizes(list);
}

/** event filter */
bool DCChat::eventFilter( QObject * object, QEvent * event )
{
	if ((event->type() == QEvent::KeyPress)&&((QTextEdit*)object==TextEdit_CHATINPUT))
	{
		QKeyEvent * e = (QKeyEvent*)event;

		bool send = FALSE;
		if ( e->state() == AltButton )
		{
			if ( (e->key() == Key_S) && (Config->GetSendChat() == "Alt + S") )
			{
				send = TRUE;
			}
			else if( ((e->key() == Key_Enter) || (e->key() == Key_Return )) &&
				   (Config->GetSendChat() == "Alt + Enter"))
			{
				send = TRUE;
			}
		}
		else if( (Config->GetSendChat() == "Ctrl + Enter") &&
			 (e->state() == ControlButton) && ( e->key() == Key_Enter || e->key() == Key_Return) )
		{
			send = TRUE;
		}
		else if( (Config->GetSendChat() == "Enter" && (e->state() != ControlButton) &&
			 ( e->key() == Key_Enter || e->key() == Key_Return)) )
		{
			send = TRUE;
		}

		if( send == TRUE )
		{
			SendMessage();
			e->accept();
			return true;
		}
		else if ( e->state() == ControlButton )
		{
			CString * ps = 0;

			// check for history
			if ( (e->key() == Key_Down) && (m_pCurrentHistory != 0) )
			{
				ps = m_History.Next(m_pCurrentHistory);
			}
			else if ( e->key() == Key_Up )
			{
				ps = m_History.Forw(m_pCurrentHistory);
			}

			if ( ps != 0 )
			{
				m_pCurrentHistory = ps;
				TextEdit_CHATINPUT->setText(m_pCurrentHistory->Data());
			}
			else if (e->key() == Key_Down)
			{
				m_pCurrentHistory = ps;
				TextEdit_CHATINPUT->clear();
			}
		}
	}
	else if ( event->type() == EVENT_TRANSLATION )
	{
		DC_TranslationEvent *e = (DC_TranslationEvent*)event;

		if ( e->m_bTranslate == TRUE )
		{
			AddStatus( tr("Translation: ") + "'" + e->m_sOriginal + "' -> '" + e->m_sTranslation + "'");
		}
		else
		{
			AddStatus( tr("Translation failed: ") + "'" + e->m_sOriginal + "'" );
		}
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** */
void DCChat::SetCrypt( bool b )
{
	if ( b )
	{
		QToolTip::remove(m_pStatusCrypt);
		QToolTip::add(m_pStatusCrypt, tr("Line is encrypted."));
		m_pStatusCrypt->setPixmap( QPixmap( encrypted_xpm ));
	}
	else
	{
		QToolTip::remove(m_pStatusCrypt);
		QToolTip::add(m_pStatusCrypt, tr("Line is not encrypted."));
		m_pStatusCrypt->setPixmap( QPixmap( decrypted_xpm ));
	}
}

/** */
void DCChat::slotClickedChatOutput( int, int )
{
	int i;
	QString pressedLink;

	pressedLink = TextEdit_CHATOUTPUT->anchorAt(TextEdit_CHATOUTPUT->mapFromGlobal(QCursor::pos()));

	for(i=0;p_LinkList[i]!=0;i++)
	{
		if ( pressedLink.left(QString(p_LinkList[i]).length()).upper() == p_LinkList[i] )
		{
			if ( Config->GetBrowser() != "" )
			{
				QProcess * pr = new QProcess();
				QByteArray buf;
				pr->addArgument(Config->GetBrowser().Data());
				pr->addArgument(pressedLink);
				pr->launch(buf);
				delete pr;
			}
		}
	}
}

/** */
void DCChat::slotRightButtonClickedChatOutput( const QPoint& )
{
	int id;

	QPopupMenu *m;
	QString s;

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiCOPY, TextEdit_CHATOUTPUT->hasSelectedText() );
	DCMenuHandler::InsertMenu( m, emiCLEAR, TRUE );
	DCMenuHandler::InsertMenu( m, emiSELECT_ALL, TRUE );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiTRANSLATE, TextEdit_CHATOUTPUT->hasSelectedText() );
	DCMenuHandler::InsertMenu( m, emiTRANSLATOR );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiREFRESH );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiSAVE );

	if ( m_bPrivateChat )
	{
		DCMenuHandler::InsertMenu( m, emiHIDE, m_bPrivateChat && !Config->GetShowChatInTab() );
		DCMenuHandler::InsertMenu( m, emiCLOSE, m_bPrivateChat );
	}

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == emiTRANSLATE )
	{
		gDCTranslator->Translate( this, m_sLanguage, TextEdit_CHATOUTPUT->selectedText() );
	}
	if ( id == emiTRANSLATOR )
	{
		m_sLanguage = gDCTranslator->SelectLanguage();
	}
	else if ( id == emiCOPY )
	{
		TextEdit_CHATOUTPUT->copy();
	}
	else if ( id == emiCLEAR )
	{
		TextEdit_CHATOUTPUT->clear();
		AddStatus( tr("Chat Cleared.") );
	}
	else if ( id == emiSELECT_ALL )
	{
		TextEdit_CHATOUTPUT->selectAll();
	}
	else if ( id == emiSAVE )
	{
		QString s = QFileDialog::getSaveFileName(
			"",
			"HTML (*.html *.htm)",
			this,
			"save file dialog",
			tr("Choose a filename to save under") );

		if ( s != "" )
		{
			QFile f(s);

			if ( f.open(IO_WriteOnly) == TRUE )
			{
				f.writeBlock(TextEdit_CHATOUTPUT->text(),TextEdit_CHATOUTPUT->text().length());
				f.close();
			}
		}
	}
	else if ( id == emiREFRESH )
	{
		if ( CFileManager::Instance()->CreateShareList() )
			s = tr("Refresh share in progress.");
		else
			s = tr("Refresh share allready in progress.");
		AddStatus(s);
	}
	else if ( id == emiCLOSE )
	{
		close();
	}
	else if ( id == emiHIDE )
	{
		setEnabled(FALSE);
		hide();
	}
}

/** */
bool DCChat::close( bool alsoDelete )
{
	// re-set the wflags ...
	setWFlags(WDestructiveClose);
	return QWidget::close(alsoDelete);
}

/** */
void DCChat::slotSendMessage()
{
	SendMessage();
}

/** */
void DCChat::SetNick( QString nick, QString hubname )
{
	sNick = nick;

	setCaption(tr("Private Chat:")+" "+nick+" ["+hubname+"]");
}

/** */
QString DCChat::GetTimeStamp()
{
	QString s = "";

	if ( ((Config->GetTimeStamp(etsPRIVATECHAT) == TRUE) && (m_bPrivateChat)) ||
	     ((Config->GetTimeStamp(etsHUBCHAT) == TRUE) && (!m_bPrivateChat)) )
	{
		s = QTime::currentTime().toString("[hh:mm:ss] ");
	}

	return s;
}

/** */
void DCChat::AddOutput( QString message )
{
	bool bscroll;

	if ( TextEdit_CHATOUTPUT->verticalScrollBar()->maxValue() == TextEdit_CHATOUTPUT->verticalScrollBar()->value() )
	{
		bscroll = TRUE;
	}
	else
	{
		bscroll = FALSE;
	}

	// convert newline
	message.replace( QRegExp("\\n\\r"), "\xa" );
	message.replace( QRegExp("\\n"), "<br>" );

	if ( (Config->GetChatMaxParagraph()!= 0) && (TextEdit_CHATOUTPUT->paragraphs() > (Config->GetChatMaxParagraph()+5)) )
	{
		TextEdit_CHATOUTPUT->removeParagraph(0);
		TextEdit_CHATOUTPUT->removeParagraph(0);
		TextEdit_CHATOUTPUT->removeParagraph(0);
		TextEdit_CHATOUTPUT->removeParagraph(0);
		TextEdit_CHATOUTPUT->removeParagraph(0);
	}

	TextEdit_CHATOUTPUT->append( message);

	if ( bscroll )
	{
		TextEdit_CHATOUTPUT->scrollToBottom();
		TextEdit_CHATOUTPUT->moveCursor( QTextEdit::MoveEnd, FALSE );
	}
}

/** */
void DCChat::AddStatus( QString message )
{
	QString timestamp;

	if ( Config->GetShowStatusMessage() == TRUE )
	{
		timestamp = GetTimeStamp();

		// convert newline
		message.replace( QRegExp("\\n\\r"), "\xa" );

#if QT_VERSION < 304
		if ( message.find("\\n",0) == -1 )
			message += "<br>";
#endif
		message.replace( QRegExp("\\n"), "<br>" );

		AddOutput( timestamp + "&lt;DCGUI&gt; " + message );
	}
}

/** */
void DCChat::AddMessage( CMessagePrivateChat * msg, bool bremote )
{
	if ( msg->m_nEncrypted != m_nEncrypted )
	{
		m_nEncrypted = msg->m_nEncrypted;
		SetCrypt((m_nEncrypted!=0));
	}

	if ( msg->sFromNick == "" )
		AddMessage( msg->sFrom.Data(), msg->sString.Data(), bremote );
	else
		AddMessage( msg->sFromNick.Data(), msg->sString.Data(), bremote );
}

/** */
void DCChat::AddMessage( QString nick, QString message, bool bremote )
{
	unsigned int len;
	bool b;
	QString msg,s,s1,timestamp;
	int index;

//	printf("ADD MESSAGE '%s'\n",message.ascii());

	if ( message == "" )
	{
		return;
	}

	// convert newline
	message.replace( QRegExp("\\r\\n"), "\xa" );
	message.replace( QRegExp("\\r"), "\xa" );

	// convert special chars ... damn hacking ;-)
	message.replace( QRegExp("<"), "&lt;" );
	message.replace( QRegExp(">"), "&gt;" );

	msg = message;
	s1 = "";

	while ( msg != "" )
	{
		index = ConvertLinks( msg, s );

		if ( index > 1 )
		{
			s1 += s;
			msg = msg.right(msg.length()-index);
			continue;
		}

		index = ConvertEmoticons( msg, s );

		if ( index > 1 )
		{
			s1 += s;
			msg = msg.right(msg.length()-index);
			continue;
		}

		if ( index == 1 )
		{
			s1 += msg.left(1);
		}

		msg = msg.right(msg.length()-index);
	}

	message = s1;

	// convert spaces
	s = "";
	b = TRUE;

	for(len=0;len<message.length();len++)
	{
		if ( message.at(len) == '\r' )
			b = TRUE;
		else if ( message.at(len) != ' ' )
			b = FALSE;

		if ( message.at(len) == ' ' )
		{
			if (b)
				s += "&nbsp;";
			else
				s += " ";
			b = !b;
		}
		else
		{
			s += message.at(len);
		}
	}

	message = s;

	// convert newline
#if QT_VERSION < 304
	if ( message.find("\\n",0) == -1 )
		message += "<br>";
#endif

	message.replace( QRegExp("\\n"), "<br>" );

	// set color
	if ( bremote && (nick != m_pClient->GetNick().Data()) )
		message = QString("<font color=\"") + Config->GetChatColor(eccREMOTETEXT).Data() + "\">" + message + "</font>";

	if ( !m_bPrivateChat )
	{
		if ( nick != m_pClient->GetNick().Data() )
			message = QString("<font color=\"") + Config->GetChatColor(eccREMOTENICK).Data() + "\">" + "<b>" + nick + "</b>: " + message + "</font>";
		else
			message = "<b>" + nick + "</b>: " + message;
	}
	else
	{
		if ( nick != m_pClient->GetNick().Data() )
			message = QString("<font color=\"") + Config->GetChatColor(eccREMOTENICK).Data() + "\">" + "<b>&lt;" + nick + "&gt;</b> " + "</font>" + message;
		else
			message = "<b>&lt;" + nick + "&gt;</b> " + message;
	}

	timestamp = GetTimeStamp();

	AddOutput( timestamp + message );

	if ( bremote && m_bPrivateChat )
	{
		Config->PlaySound(eusRECEIVE);

		// send away message
		if ( Config->GetAwayMode() == euamAWAY )
		{
			s = Config->GetAwayMessage().Data();

			if ( (s != "") && (bSendAway) )
			{
				m_pClient->SendPrivateMessage( m_pClient->GetNick(), sNick.ascii(), s.ascii() );
				bSendAway = FALSE;
			}
		}
		else
		{
			bSendAway = TRUE;
		}
	}
}

/** */
void DCChat::SendMessage()
{
	int err;
	QString message;
	CString *ps;

	message = TextEdit_CHATINPUT->text();

	if ( message != "" )
	{
		// add history
		if ( m_History.Count() == 10 )
		{
			ps = m_History.Next(0);
			m_History.Del(ps);
		}

		m_pCurrentHistory = 0;
		m_History.Add(new CString(message.ascii()));

		if ( (CheckForCommand() == FALSE) &&
		     (m_pClient) )
		{
			// convert the newlines
			message.replace( QRegExp("\\r\\n"), "\xa" );
			message.replace( QRegExp("\\n"), "\xd\xa" );

			if ( m_bPrivateChat )
				err = m_pClient->SendPrivateMessage( m_pClient->GetNick(), sNick.ascii(), message.ascii() );
			else
				err = m_pClient->SendChat( m_pClient->GetNick(), message.ascii() );

			if ( err == 0 )
			{
				if ( m_bPrivateChat )
					AddMessage( m_pClient->GetNick().Data(), message.ascii(), FALSE );
				Config->PlaySound(eusSEND);

				TextEdit_CHATINPUT->setText("");
			}
			else
			{
				AddStatus( tr("Message not send !") );
			}
		}
		else
		{
			TextEdit_CHATINPUT->setText("");
		}
	}
}

/** */
int DCChat::ConvertLinks( QString msg, QString & s )
{
	int ret = 1;
	int i,i1;

	// convert links
	for(i=0;p_LinkList[i]!=0;i++)
	{
		if ( msg.left(strlen(p_LinkList[i])).upper() == p_LinkList[i] )
		{
			// link end at space or newline
			if ( (i1 = msg.find(" ",0,FALSE)) == -1 )
					i1 = msg.find("\xa",0,FALSE);

			if ( i1 == -1 )
				i1 = msg.length();

			s = msg.left(i1);
			s = "<a name=\""+s+"\" href=\""+s+"\">"+s+"</a>";

			ret = i1;

			break;
		}
	}

	return ret;
}

/** */
int DCChat::ConvertEmoticons( QString msg, QString & s )
{
	int ret = 1;
	QPtrList<DC_EmoticonObject> * elist;

	elist = Config->EmoticonList();

	// convert emoticons
	if ( (Config->GetEnableEmoticons() == TRUE) && (elist != 0) )
	{
		DC_EmoticonObject * EmoticonObject;

		for(EmoticonObject=elist->first();EmoticonObject;EmoticonObject=elist->next())
		{
			if ( msg.left(EmoticonObject->m_Text.length()) == EmoticonObject->m_Text )
			{
				s   = "<img source=\""+QString("emoticon")+QString().setNum(EmoticonObject->m_nID)+"\">";
				ret = EmoticonObject->m_Text.length();
				break;
			}
		}
	}

	return ret;
}

/** */
bool DCChat::CheckForCommand()
{
	bool res = FALSE;
	QString timestamp;
	QString cmd,s;

	cmd = TextEdit_CHATINPUT->text();


	if ( cmd != "" )
	{
		// help
		if ( cmd == "/dchelp" )
		{
			QString s;

			s  = tr("Help:") + "\n";
			s += tr("/clear - clear the chat window") + "\n";
			s += tr("/mode - show current mode") + "\n";
			s += tr("/refresh - refresh share") + "\n";
			s += tr("/ts - switch time display in chat on/off") + "\n";
			s += tr("/dcgui - send an advertisment to the hub") + "\n";
			s += tr("/join &lt;address&gt; - disconnect from currently connected hub and connect to another hub") + "\n";
			s += tr("/msg &lt;nick&gt; &lt;message&gt; - send private message") + "\n";
			s += tr("/away &lt;message&gt; - set automatic response for private messages; if the message is empty, disable it");

			AddStatus(s);
			res = TRUE;
		}
		// /clear - clear the chat window
		else if ( cmd == "/clear" )
		{
			TextEdit_CHATOUTPUT->clear();

			AddStatus( tr("Chat Cleared.") );
			res = TRUE;
		}
		// /mode - show current mode
		else if ( cmd == "/mode" )
		{
			s  = tr("Current mode: ");

			if ( Config->GetMode() == ecmActive )
				s += tr("active");
			else
				s += tr("passive");
			s += ".";

			AddStatus(s);
			res = TRUE;
		}
		// /refresh - refresh share 
		else if ( cmd == "/refresh" )
		{
			if ( CFileManager::Instance()->CreateShareList() == TRUE )
			{
				s = tr("Refresh share in progress.");
			}
			else
			{
				s = tr("Refresh share allready in progress.");
			}

			AddStatus(s);
			res = TRUE;
		}
		// /ts - switch time display in chat on/off 
		else if ( cmd == "/ts" )
		{
			bool b;

			if ( m_bPrivateChat )
			{
				b = !Config->GetTimeStamp(etsPRIVATECHAT);
				Config->SetTimeStamp(etsPRIVATECHAT,b);
			}
			else
			{
				b = !Config->GetTimeStamp(etsHUBCHAT);
				Config->SetTimeStamp(etsHUBCHAT,b);
			}

			s = tr("Switch timestamp ");

			if ( b )
				s += "on.";
			else
				s += "off.";

			AddStatus(s);
			res = TRUE;
		}
		// /dcgui - send an advertisment to the hub
		else if ( cmd == "/dcgui" )
		{
			AddStatus( tr("Send advertisment.") );
			res = TRUE;
		}
		// /join <address> - disconnect from currently connected hub and connect to another hub 
		else if ( cmd.find("/join ",0,FALSE) == 0 )
		{
			QString address;

			address = cmd.section(' ',1,1);

			if ( address == "" )
				s = tr("Wrong parameter for '/join'");
			else
			{
				s = tr("Join to: ") + address;
				m_pClient->SetHubName(address.ascii());
				m_pClient->setCaption(address);
				m_pClient->Connect(address.ascii());
			}

			AddStatus(s);
			res = TRUE;
		}
		// /msg <nick> <message> - send private message 
		else if ( cmd.find("/msg ",0,FALSE) == 0 )
		{
			int i;
			QString nick,message="";

			nick = cmd.section(' ',1,1);

			i = cmd.find(' ',5);

			if ( i != -1 )
				message = cmd.mid(i+1);

			if ( (nick == "") || (message == "") )
			{
				s = tr("Wrong parameter for '/msg'");
			}
			else
			{
				if ( m_pClient->SendPrivateMessage( m_pClient->GetNick(), nick.ascii(), message.ascii() ) == 0 )
				{
					s = tr("Private message send.");
					Config->PlaySound(eusSEND);
				}
				else
				{
					s = tr("Private message not send !");
				}
			}

			AddStatus(s);
			res = TRUE;

		}
		// /away <message> - set automatic response for private messages; if the message is empty, disable it 
		else if ( cmd.find("/away",0,FALSE) == 0 )
		{
			int i;
			QString message="";

			i = cmd.find(' ',5);

			if ( i > 0 )
				message = cmd.mid(i+1);

			Config->SetAwayMessage(message.ascii());

			if ( message == "" )
				s = tr("Away message disabled.");
			else
				s = tr("Away message set.");

			AddStatus( s );
			res = TRUE;
		}
	}
	return res;
}
