/***************************************************************************
                          dcfilebrowser.cpp  -  description
                             -------------------
    begin                : Fre Nov 29 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qregexp.h>
#include <qheader.h>
#include <qlabel.h>

#include <dcfiletool.h>
#include <dcwidget.h>

#include "dcfilebrowser.h"

#include <dclib/dcos.h>
#include <dclib/cstring.h>
#include <dclib/cdir.h>

#include <res/folder_red.xpm>
#include <res/folder_blue.xpm>
#include <res/folder_blue_open.xpm>

/** */
DCFileBrowser::DCFileBrowser(QWidget *parent, const char *name ) : DCDialogFileBrowser(parent,name)
{
	// set width mode
	ListView_DIRECTORY->setColumnWidthMode( 0, QListView::Maximum );
	// hide the column header
	ListView_DIRECTORY->header()->hide();

	InitDocument();
}

/** */
DCFileBrowser::~DCFileBrowser()
{
}
                                                                                   
void DCFileBrowser::InitDocument()
{
	connect( ListView_DIRECTORY, SIGNAL(currentChanged(QListViewItem*)), this, SLOT(slotCurrentChangedDirectory(QListViewItem*)) );
	connect( ListView_DIRECTORY, SIGNAL(expanded(QListViewItem*)), this, SLOT(slotExpandedDirectory(QListViewItem*)) );
	connect( ListView_DIRECTORY, SIGNAL(collapsed(QListViewItem*)), this, SLOT(slotCollapsedDirectory(QListViewItem*)) );
}

/** */
void DCFileBrowser::slotCurrentChangedDirectory( QListViewItem * item )
{
	DCFileItem * FileItem;
	CDir dir;
	QString s;
	ulonglong size=0;
	int items = 0;

	if ( item == 0 )
		return;

	ListView_FILES->clear();

	for ( FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.first(); FileItem; FileItem = ((DCFileBrowserListItem*)item)->m_pFileList.next() )
	{
		size += CString(FileItem->m_sSize.ascii()).asULL();
		s = dir.Extension(FileItem->m_sName.ascii()).Data();

		if ( s != "" )
			s += " " + tr("file");
		items++;

		DC_QListViewItem *item1 = new DC_QListViewItem(ListView_FILES);
		item1->myvalue = CString(FileItem->m_sSize.ascii()).asULL();
		item1->mycol   = 1;

		item1->setText(0,FileItem->m_sName);
		item1->setText(1,FileItem->m_sSize);
		item1->setText(2,s);
	}

	TextLabel_STATUS->setText( QString().setNum(items+item->childCount()) + " " + tr("Items") + " - " + 
					QString().setNum(items) + " " + tr("Files") +
					" (" + DCFileTool::GetSizeString(size).Data() + " " + tr("Total") + ") - " +
					QString().setNum(item->childCount()) + " " + tr("Directories") );
}

/** */
void DCFileBrowser::slotExpandedDirectory( QListViewItem * item )
{
	if ( item->parent() != NULL )
		item->setPixmap(0,QPixmap(folder_blue_open_xpm));
}

/** */
void DCFileBrowser::slotCollapsedDirectory( QListViewItem * item )
{
	if ( item->parent() != NULL )
		item->setPixmap(0,QPixmap(folder_blue_xpm));
}

/** */
void DCFileBrowser::InitTree( QString nick, QString hubname, QString hubhost, QString s )
{
	QListViewItem * item;
	QString s1,sname,slength;
	long i,j,d;
	int depth;
	ulonglong filecount;
	ulonglong shared;

	m_sNick    = nick;
	m_sHubName = hubname;
	m_sHubHost = hubhost;

	i = j = 0;
	filecount = 0;
	shared = 0;
	item = 0;

	ListView_DIRECTORY->setUpdatesEnabled(FALSE);

	item = new DCFileBrowserListItem( ListView_DIRECTORY );
	item->setText(0,tr("Root Directory"));
	item->setPixmap(0,QPixmap(folder_red_xpm));

	while((i=s.find("\xD\xA",j))>0)
	{
		s1 = s.mid(j,i-j);

		depth=0;
		while( (s1.find("\x9",depth)) != -1 )
			depth++;
		s1 = s1.replace( QRegExp("\x9"), "" );

		if (item)
			while ( depth < item->depth() )
				if ( (item=item->parent()) == 0 )
					break;

		if ( s1 != "" )
		{
			d = s1.findRev("|");

			if ( d != -1 ) // parse file entry
			{
				filecount++;

				sname   = s1.mid(0,d);
				slength = s1.right(s1.length()-d-1);

				if ( slength == "" )
					slength = tr("<wrong length>");
				else
					shared += CString(slength.ascii()).asULL();

				DCFileItem * FileItem = new DCFileItem();
				FileItem->m_sName = sname;
				FileItem->m_sSize = slength;

				((DCFileBrowserListItem*)item)->m_pFileList.append(FileItem);
//				if (item==0)
//					item1 = new DCFileBrowserListItem( ListView_DIRECTORY );
//				else
//					item1 = new DCFileBrowserListItem( item );
//
//				item1->setText(0,sname);
//				item1->setText(0,slength);
			}
			else // parse dir entry
			{
				if (item==0)
					item = new DCFileBrowserListItem( ListView_DIRECTORY );
				else
					item = new DCFileBrowserListItem( item );

				item->setText(0,s1);
				item->setPixmap(0,QPixmap(folder_blue_xpm));
			}
		}

		j = i+2;
	}

//	LineEdit_FILES->setText( CString().setNum(filecount).Data() );
//	LineEdit_SHARED->setText( DCFileTool::GetSizeString(shared).Data() );

	ListView_DIRECTORY->setUpdatesEnabled(TRUE);
	ListView_DIRECTORY->triggerUpdate();
}
