/***************************************************************************
                          dcfiletool.cpp  -  description
                             -------------------
    begin                : Die Mr 26 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qstring.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qdir.h>
#include <qregexp.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include <dclib/cdir.h>

#include <dcconfig.h>
#include <dctransferview.h>
#include <dcserverlist.h>

#include "dcfiletool.h"

#include <dclib/dcos.h>

/** */
DCFileTool::DCFileTool()
{
	Init();
}

/** */
DCFileTool::~DCFileTool()
{
}

/** */
void DCFileTool::Init()
{
	iLastDownloadCase = -1;
	bCaseAllFiles     = FALSE;
}

/** */
bool DCFileTool::SelectFileSource( ulonglong size, QString & file, QString & localrootpath, QString & localpath )
{
	bool res = FALSE;
	QStringVector LocalFilesList;
	int id;

	pTransferView->GetLocalFilesList( LocalFilesList, size );

	if ( LocalFilesList.empty() )
	{
		return res;
	}

	// Add as an extra source
	QPopupMenu * m = new QPopupMenu;

	for ( int iter=0; iter<(int)LocalFilesList.size(); iter++ )
	{
		if ( LocalFilesList[iter].contains("\\") )
		{
			m->insertItem( LocalFilesList[iter].section("\\",-1,-1), iter );
		}
		else
		{
			m->insertItem( LocalFilesList[iter].section("/",-1,-1), iter );
		}
	}

	m->insertItem( tr("(choose an existing download to add to)"), 999);
	m->setItemEnabled(999,false);

	id = m->exec(QCursor::pos());

	delete m;

	if ( id != -1 )
	{
		QFileInfo fi(LocalFilesList[id]);

		file          = fi.fileName();
		localpath     = "";
		localrootpath = fi.dirPath();

		// now we must check if a localpath exist
		if ( localrootpath.find( Config->GetDownloadFolder().Data(), 0 ) == 0 )
		{
			localpath     = localrootpath.mid( Config->GetDownloadFolder().Length(), localrootpath.length()-Config->GetDownloadFolder().Length() );
			localrootpath = Config->GetDownloadFolder().Data();
		}

		res = TRUE;
	}

	return res;
}

/** */
bool DCFileTool::AddFileSource( CString nick, CString hubname, CString hubhost,
				CString remotename, CString localname,
				CString localpath, CString localrootpath,
				eltMedium medium, ulonglong size )
{
	bool res = FALSE;
	QString sourcefile = "",slocalrootpath,slocalpath;

	if ( SelectFileSource( size, sourcefile, slocalrootpath, slocalpath ) == FALSE )
	{
		return CheckFile( nick, hubname, hubhost,
				remotename, localname,
				localpath, localrootpath,
				medium, size );
	}

	if ( !sourcefile.isEmpty() )
	{
		// add transfer to the waitlist
		res = CheckFile( nick, hubname, hubhost,
				remotename, sourcefile.ascii(),
				slocalpath.ascii(), slocalrootpath.ascii(),
				medium, size, TRUE );

	}

	return res;
}

/** */
bool DCFileTool::CheckFile( CString nick, CString hubname, CString hubhost,
				CString remotename, CString localname,
				CString localpath, CString localrootpath,
				eltMedium medium, ulonglong size, bool usermulti )
{
	CDir dir;
	QString name;
	ulonglong fsize,fpos=0;
	int filestate,i;
	bool multi = usermulti;

	// first we get the filestate from the downloadmanager
	filestate = pTransferView->DLM_QueueCheck( nick, hubname, hubhost,
			remotename, localname,
			localpath, localrootpath,
			medium, size );

	if ( filestate == 1 )
	{
//		QMessageBox::information( this, tr("File download"),
//			tr("The same user/file is allready in the queue !\n"));
		return FALSE;
	}
	else if ( (filestate == 2) && (multi == FALSE) )
	{
		switch( QMessageBox::warning( this, tr("File download"),
			tr("A same file is allready in the queue !"),
		        tr("Start Multi Download"),
			tr("Cancel"), 0, 1 ) )
		{
			case 0:
				multi = TRUE;
		        	break;
			default:
				return FALSE;
				break;
		}
	}
	else if ( filestate == 3 )
	{
		QMessageBox::information( this, tr("File download"),
			tr("File allready in the queue and not mark as multi download !\n"));
		return FALSE;
	}
	else if ( filestate == 4 )
	{
		QMessageBox::information( this, tr("File download"),
			tr("File allready in the queue with a different size !\n"));
		return FALSE;
	}
	else if ( filestate == 0 )
	{
		if ( localrootpath != "" )
		{
			name = localrootpath.Data();
		}
		else
		{
			name = Config->GetDownloadFolder().Data();
		}

		if ( name == "" )
		{
			return FALSE;
		}

		name = name + "/" + localpath.Data() + "/" + localname.Data();

		QFile f(name);

		fsize = dir.getFileSize(name.ascii(),FALSE);

		if ( fsize == 0 )
		{
			fpos = 0;

			if ( size > (1024*1024) )
			{
				if ( (Config->GetDefaultDownloadMode() == 0) && (multi == FALSE) )
				{
					if ( bCaseAllFiles == FALSE )
					{
						i = QMessageBox::warning( this, tr("File download"),
									  tr("You can start a Multi Download !"),
									  QMessageBox::Yes | QMessageBox::Default,
									  QMessageBox::No | QMessageBox::Escape,
									  0 );

						if ( iLastDownloadCase == -1 )
						{
							switch ( QMessageBox::warning( this, tr("File download"),
										       tr("If you want to download all files in this mode ?"),
										       QMessageBox::Yes | QMessageBox::Default,
										       QMessageBox::No | QMessageBox::Escape,
										       0 ) )
							{
							case QMessageBox::Yes:
								bCaseAllFiles = TRUE;
						        	break;
							default:
								break;
							}
						}

						iLastDownloadCase = i;
					}

					switch( iLastDownloadCase )
					{
						case QMessageBox::Yes:
							multi = TRUE;
				       		 	break;
						default:
							break;
					}
				}
				else if ( (Config->GetDefaultDownloadMode() == 1) && (multi == FALSE) )
				{
					multi=FALSE;
				}
				else
				{
					multi=TRUE;
				}
			}
			else
			{
				multi = FALSE;
			}
		}
		else if ( size == fsize )
		{
			QMessageBox::information( this, tr("File download"),
				tr("File allready download\n"));
			return FALSE;
		}
		else if ( fsize > 0 )
		{
			switch( QMessageBox::warning( this, tr("File download"),
				tr("File allready exist !"),
			        tr("Resume"),
			        tr("Overwrite"),
				tr("Cancel"), 0) )
			{
				case 0:
					fpos = fsize;
			        	break;
				case 1:
					f.remove();
					fpos = 0;
		        		break;
				case 2:
					return FALSE;
					break;
			}
		}
		else
		{
			return FALSE;
		}
	}

	if ( pServerList->IsHubOnline(hubname) == ehsNONE )
	{
		switch( QMessageBox::warning( this, tr("File download"),
					tr("Not connected to required hub!"),
					tr("Connect"),
					tr("Cancel"), 0, 0, 1 ) )
		{
			case 0:
				pServerList->Connect(hubname.Data(),hubhost.Data());
				break;
			case 1:
				break;
		}
	}

	pTransferView->DLM_QueueAdd( nick, hubname, hubhost,
				remotename, localname,
				localpath, localrootpath,
				medium, size, fpos, multi );

	return TRUE;
}

/** */
CString DCFileTool::GetSizeString( const ulonglong size )
{
	CString s, suffix;
	double f;
	eUnit unit;

	unit = Config->GetUnit();

	if ( ((size > (1024*1024*1024)) && (unit == euAUTO)) || (unit == euGBYTE) )
	{
		suffix = "GB";
#ifndef WIN32
		f = (size*1.0)/(1024*1024*1024);
#else
		f = ((longlong)size*1.0)/(1024*1024*1024);
#endif
	}
	else if ( ((size > (1024*1024)) && (unit == euAUTO)) || (unit == euMBYTE) )
	{
		suffix = "MB";
#ifndef WIN32
		f = (size*1.0)/(1024*1024);
#else
		f = ((longlong)size*1.0)/(1024*1024);
#endif
	}
	else if ( ((size > 1024) && (unit == euAUTO)) || (unit == euKBYTE) )
	{
		suffix = "KB";
#ifndef WIN32
		f = (size*1.0)/1024;
#else
		f = ((longlong)size*1.0)/1024;
#endif
	}
	else if  ( (unit == euAUTO) || (unit == euBYTE) )
	{
		suffix = "B";
		f = 0;
	}
	else
	{
		f = 0;
	}

	if ( f != 0 )
	{
		s = QString().setNum(f,'f',2).ascii();
	}
	else
	{
		s = s.setNum(size);
	}

	s += " " + suffix;

	return s;
}


/* This will Convert Total Secs to how much time is
 * remaining X Days + X Hrs + X Mins + X Secs */
CString DCFileTool::GetTimeString( const ulonglong sec )
{
	CString s,r;
	ulonglong rate;

	rate = sec;

	r = " S";
	s = "";

	if ( rate > (60*60*24) )
	{
		s+= CString().setNum(rate/(60*60*24)) +" D ";
		rate %= (60*60*24);
	}
	if ( rate > (60*60) )
	{
		s+= CString().setNum(rate/(60*60)).RightJustify(2,'0')+":";
		rate %= (60*60);
	}
	else
	{
		s+= "00:";
	}
	if ( rate > 60 )
	{
		s += CString().setNum(rate/60).RightJustify(2,'0')+":";
		rate %= 60;
	}
	else
	{
		s+= "00:";
	}
	s += CString().setNum(rate).RightJustify(2,'0').RightJustify(2,'0');

	return s;
}

/** */
CString DCFileTool::GetTransferString( const ulonglong rate )
{
	CString s;
	ulonglong r;

	r = rate;

	s = "B/s";
	if ( r >= 1024 )
	{
		s = "KB/s";
		r/=1024;
	}
	else if ( r >= (1024*1024) )
	{
		s = "MB/s";
		r/=(1024*1024);
	}

	s = CString().setNum(r) + s;

	return s;
}
