/***************************************************************************
                          dchubsearch.cpp  -  description
                             -------------------
    begin                : Fri Mar 15 2002
    copyright            : (C) 2002 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapp.h>
#include <qlineedit.h>
#include <qtextedit.h>
#include <qradiobutton.h>
#include <qlistview.h>
#include <qcheckbox.h>
#include <qlcdnumber.h>
#include <qcombobox.h>
#include <qregexp.h>
#include <qspinbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qbuttongroup.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qfiledialog.h>
#include <qclipboard.h>
#include <qprogressbar.h>
#include <qvaluevector.h>
//#include <qdragobject.h>
#include <qtoolbutton.h>
#include <qinputdialog.h>

#ifndef WIN32
#include <stdlib.h>
#endif

#include <dcconfig.h>
#include <dcclient.h>
#include <dcserverlist.h>
#include <dcfiletool.h>
#include <dctransferview.h>
#include <dcmenuhandler.h>
#include <dcwidget.h>

#include "dchubsearch.h"

#include <dclib/dcos.h>
#include <dclib/cdir.h>

#include "res/find.xpm"

DCHubSearch * pHubSearch = 0;

/** */
DCHubSearch::DCHubSearch(QWidget* parent, const char *name, int wflags) : DCDialogHubSearch(parent, name, wflags)
{
	int idx;

	// set default icon
	setIcon( QPixmap( find_xpm ) );

	pWorkspace = (QWorkspace*)parent;

	// set the ColumnWidthMode to manual, we will take care of this task
	for( idx = 0; idx < ListView_SEARCHRESULT->columns(); idx++ )
	{
		ListView_SEARCHRESULT->setColumnWidthMode( idx, QListView::Manual );
	}

	for( idx = 0; idx < ListView_SEARCHRESULTUSER->columns(); idx++ )
	{
		ListView_SEARCHRESULTUSER->setColumnWidthMode( idx, QListView::Manual );
	}

	ehSearchType = ehstNONE;

	m_pMessageList      = new CList<CObject>();
	m_pSearchResultList = new CList<CObject>();
	m_pSearchQueryList  = 0;

	InitDocument();

	pHubSearch = this;
}

/** */
DCHubSearch::~DCHubSearch()
{
	ehSearchType = ehstNONE;

	Disconnect();

	Timer.stop();

	// clear the messagelist
	SocketCallbackThread.Lock();

	if ( m_pMessageList )
	{
		delete m_pMessageList;
		m_pMessageList = 0;
	}

	if ( m_pSearchResultList )
	{
		delete m_pSearchResultList;
		m_pSearchResultList = 0;
	}

	SocketCallbackThread.UnLock();
}

/** */
void DCHubSearch::InitDocument()
{
	StringMap * map;

	// restore settings
	if ( Config->GetMap("SEARCHVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGTH"].toInt() > 0) )
		{
			setGeometry( (*map)["X"].toInt(), (*map)["Y"].toInt(), (*map)["WIDTH"].toInt(), (*map)["HEIGTH"].toInt() );
		}
	}

	connect( LineEdit_SEARCH, SIGNAL(returnPressed()), this, SLOT(slotSearchReturnPressed()) );
	connect( PushButton_SEARCH, SIGNAL(clicked()), this, SLOT(slotSearchReturnPressed()) );
	connect( PushButton_STOP, SIGNAL(clicked()), this, SLOT(slotClickedStop()) );
	connect( PushButton_REFRESHCONNECTEDHUBS, SIGNAL(clicked()), this, SLOT(slotRefreshConnectedHubs()) );
	connect( ToolButton_HISTORY, SIGNAL(clicked()), this, SLOT(slotClickedHistory()) );
	connect( SpinBox_FREESLOTS, SIGNAL(valueChanged(int)), this, SLOT(slotChangedFreeSlots(int)) );
	connect( CheckBox_SEARCHFILEONLY, SIGNAL(toggled(bool)), this, SLOT(slotToggledSearchFileOnly(bool)) );
	connect( TabWidget_HUBSEARCH,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetCurrentChange(QWidget*)) );
	connect( ListView_SEARCHRESULT,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClickedSearchResult(QListViewItem*)) );
//	connect( ListView_SEARCHRESULT,SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedSearchResult(QListViewItem*, const QPoint &, int )) );
	connect( ListView_SEARCHRESULT,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedSearchResult(QListViewItem*, const QPoint &, int )) );
	connect( ToolButton_RESETFILTER, SIGNAL(clicked()), this, SLOT(slotReset()) );
	connect( ToolButton_RESETFILTER, SIGNAL(clicked()), this, SLOT(slotReset()) );
	connect( LineEdit_INCLUDE, SIGNAL(returnPressed()), this, SLOT(slotTextFilterResults()) );
	connect( LineEdit_EXCLUDE, SIGNAL(returnPressed()), this, SLOT(slotTextFilterResults()) );
	connect( PushButton_APPLYTEXTFILTER, SIGNAL(clicked()), this, SLOT(slotTextFilterResults()) );
	connect( PushButton_RESETTEXTFILTER, SIGNAL(clicked()), this, SLOT(slotTextFilterReset()) );
	
	connect( &Timer, SIGNAL(timeout()), this, SLOT(timerDone()) );
/*
	CMessageSearchResult * MessageSearchResult;
	int i;

	for(i=0;i<1000;i++) {
		MessageSearchResult = new CMessageSearchResult();
		ShowSearchResult( MessageSearchResult, ListView_SEARCHRESULT );
		m_pSearchResultList->Add(MessageSearchResult);
	}
*/
}

/** */
void DCHubSearch::DeInitDocument()
{
	StringMap * map;

	// save search view settings
	Config->GetMap("SEARCHVIEW",map);

	(*map)["X"]         = QString().setNum(x());
	(*map)["Y"]         = QString().setNum(y());
	(*map)["WIDTH"]     = QString().setNum(width());
	(*map)["HEIGTH"]    = QString().setNum(height());
	(*map)["VISIBLE"]   = QString().setNum(isVisible());
	(*map)["MAXIMIZED"] = QString().setNum(isMaximized());
	(*map)["MINIMIZED"] = QString().setNum(isMinimized());
	(*map)["ENABLED"]   = QString().setNum(isEnabled());
}

/** current tab widget change slot */
void DCHubSearch::slotTabWidgetCurrentChange(QWidget*)
{
	ResizeListViewColumn();
}

/** overridden so that the columns are resized on show() */
void DCHubSearch::show()
{
	QWidget::show();

	ResizeListViewColumn();
}

/** resize event handler */
void DCHubSearch::resizeEvent( QResizeEvent * )
{
	ResizeListViewColumn();
}

/** resize the ListView columns */
void DCHubSearch::ResizeListViewColumn()
{
	int width;

	if ( ListView_SEARCHRESULT->isVisible() )
	{
		width = ListView_SEARCHRESULT->width();

		ListView_SEARCHRESULT->setColumnWidth( 0, ((width*7)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 1, ((width*3)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 2, ((width*4)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 3, ((width*1)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 4, ((width*4)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 5, ((width*6)/28) );
		ListView_SEARCHRESULT->setColumnWidth( 6, ((width*3)/28) );
	}

	if ( ListView_SEARCHRESULTUSER->isVisible() )
	{
		width = ListView_SEARCHRESULTUSER->width();

		ListView_SEARCHRESULTUSER->setColumnWidth( 0, ((width)/2) );
		ListView_SEARCHRESULTUSER->setColumnWidth( 1, ((width)/2) );
	}
}

/** */
int DCHubSearch::selectedItems( QListView * list, QPtrList<QListViewItem> & lst )
{
	QListViewItemIterator it( (QListView *)list );

	for ( ; it.current(); it++ )
	{
		if ( it.current()->isSelected() )
		{
			lst.append(it.current());
		}
	}

	return lst.count();
}

/** */
void DCHubSearch::SetSearchView( bool enabled )
{
	PushButton_SEARCH->setEnabled(enabled);
	LineEdit_SEARCH->setEnabled(enabled);
	ButtonGroup_HUBS->setEnabled(enabled);
	//	always enabled, because they are "live" filters
	//	GroupBox_FILTER->setEnabled(enabled);
	SpinBox_MAXTHREADS->setEnabled(enabled);
	ToolButton_HISTORY->setEnabled(enabled);
	CheckBox_SEARCHUSER->setEnabled(enabled);
	PushButton_STOP->setEnabled(!enabled);
}

/** */
int DCHubSearch::DC_CallBack( CObject * Object )
{
	SocketCallbackThread.Lock();

	int err = -1;

	if ( ehSearchType != ehstNONE )
	{
		if ( (Object != 0) && (m_pMessageList != 0) )
		{
			m_pMessageList->Add(Object);
			err = 0;
		}
        }

	SocketCallbackThread.UnLock();

	return err;
}

/** */
void DCHubSearch::timerDone()
{
	int i,t;

	if ( ehSearchType != ehstNONE )
	{
		ShowResults(FALSE);
	}

	qApp->processEvents();

	// search is done
	if ( ehSearchType == ehstNONE )
	{
		// update view
		SetSearchView( TRUE );
	}

	if ( pHubServerList != 0 )
	{
		LineEdit_LOGHUBS->setText( QString().setNum(iCurrentHubServer)+"/"+QString().setNum(pHubServerList->Count()) );

		i=0;
		if (pHubServerList->Count()>0)
			i = ((iCurrentHubServer*100)/pHubServerList->Count());
		ProgressBar_LOGHUBS->setProgress(i);

		i = time(0)-tStartTime;

		if ( iCurrentHubServer > 0 )
			t = ((i*pHubServerList->Count())/iCurrentHubServer);
		else
			t = 0;

		LineEdit_LOGTIME->setText( (DCFileTool::GetTimeString(i) + "/" + DCFileTool::GetTimeString(t)).Data() );
	}

	if ( ehSearchType != ehstNONE )
	{
		// restart timer
		Timer.start( 500, TRUE );
	}
}

/** */
void DCHubSearch::ShowResults( bool bClearList )
{
	CDCMessage *DCMsg;
	CObject * Object;
	bool bupdate;

	if ( bClearList == TRUE )
	{
		ListView_SEARCHRESULT->clear();
	}

	if ( SocketCallbackThread.TryLock() == FALSE )
	{
		return;
	}

	bupdate = FALSE;

	if ( m_pMessageList != 0 )
	{
		while( (Object = m_pMessageList->Next(0)) != 0 )
		{
			m_pMessageList->Remove(Object);

			DCMsg = (CDCMessage*) Object;

			switch ( DCMsg->m_eType )
			{
				case DC_MESSAGE_SEARCHRESULT:
				{
					bool b;
					b = DC_SearchResult( (CMessageSearchResult *) Object );

					if( b == TRUE )
					{
						Object  = 0;
						bupdate = TRUE;
						ListView_SEARCHRESULT->setUpdatesEnabled(FALSE);
					}

					break;
				}

				case DC_MESSAGE_SEARCHRESULT_USER:
				{
					CMessageSearchResultUser * msg = (CMessageSearchResultUser*) Object;

					QListViewItem *item = new QListViewItem(ListView_SEARCHRESULTUSER);
					item->setText(0,msg->sNick.Data());
					item->setText(1,msg->m_sHubName.Data());
					break;
				}

				case DC_MESSAGE_LOG:
				{
					DC_LogMessage( (CMessageLog *) Object );

					break;
				}

				default:
				{
					break;
				}
			}

			if ( Object )
				delete Object;
		}
	}

	SocketCallbackThread.UnLock();

	if ( bupdate == TRUE )
	{
		ListView_SEARCHRESULT->setUpdatesEnabled(TRUE);
		ListView_SEARCHRESULT->triggerUpdate();
	}
}

/** search result */
bool DCHubSearch::DC_SearchResult( CMessageSearchResult * MessageSearchResult )
{
	bool res = FALSE;
	ulonglong size;

	if ( ehSearchType == ehstNONE )
	{
		return res;
	}

	if ( SpinBox_MAXRESULT->value() > 0 )
	{
		if ( ListView_SEARCHRESULT->childCount() >= SpinBox_MAXRESULT->value() )
		{
			return res;
		}
	}

	size = SpinBox_SEARCHSIZE->value();

	if ( ComboBox_SEARCHUNIT->currentItem() == 1 )
	{
		size *= 1024;
	}
	else if ( ComboBox_SEARCHUNIT->currentItem() == 2 )
	{
		size *= 1024*1024;
	}
	else if ( ComboBox_SEARCHUNIT->currentItem() == 3 )
	{
		size *= 1024*1024*1024;
	}

	if ( ComboBox_SEARCHLIMIT->currentItem() == 0 )
	{
		if ( MessageSearchResult->lSize < size )
		{
			return res;
		}
	}
	else if ( ComboBox_SEARCHLIMIT->currentItem() == 1 )
	{
		if ( MessageSearchResult->lSize > size )
		{
			return res;
		}
	}
	else if ( ComboBox_SEARCHLIMIT->currentItem() == 2 )
	{
		if ( MessageSearchResult->lSize != size )
		{
			return res;
		}
	}

	ShowSearchResult( MessageSearchResult, ListView_SEARCHRESULT );

	res = TRUE;

	m_pSearchResultList->Add(MessageSearchResult);

	LCDNumber_RESULTS->display(ListView_SEARCHRESULT->childCount());

	return res;
}

/** */
QListViewItem * DCHubSearch::ShowSearchResult( CMessageSearchResult * MessageSearchResult, QListView * parent )
{
	DC_ListResult *item = new DC_ListResult(parent);

	AddSearchResult( MessageSearchResult, item );

	return item;
}

/** */
QListViewItem * DCHubSearch::ShowSearchResult( CMessageSearchResult * MessageSearchResult, DC_ListResult * parent )
{
	DC_ListResult *item = new DC_ListResult(parent);

	AddSearchResult( MessageSearchResult, item );

	return item;
}

/** */
bool DCHubSearch::AddSearchResult( CMessageSearchResult * MessageSearchResult, DC_ListResult * item )
{
	bool res = FALSE;
	bool visible = TRUE;
	int i;
	QString s;
	CString spath,sname;
	DCFileTool FileTool;

	if ( MessageSearchResult->iFreeSlot < (unsigned int)SpinBox_FREESLOTS->value() )
	{
		visible = FALSE;
	}

	s = MessageSearchResult->sFile.Data();

	if (CheckBox_SEARCHFILEONLY->isChecked())
	{
	        // Checks if s matches the search pattern
		QString search_pattern;
		for(i=0;i<=LineEdit_SEARCH->text().contains(" ");i++)
		{
		        search_pattern = LineEdit_SEARCH->text().section(" ",i,i);

			if ( s.find(search_pattern,0,FALSE) == -1 )
			{
				visible = FALSE;
			}
		}
	}

	// split filename and path
	s = MessageSearchResult->sFile.Data();

	CDir().SplitPathFile(s.ascii(),spath,sname);

	item->myvalue = MessageSearchResult->lSize;
	item->mycol   = 1;

	item->setText(0,sname.Data());
	item->setText(1,FileTool.GetSizeString(MessageSearchResult->lSize).Data());
	item->setText(2,MessageSearchResult->sNick.Data());
	item->setText(3,QString().setNum(MessageSearchResult->iFreeSlot)+'/'+QString().setNum(MessageSearchResult->iMaxSlot));
	item->setText(4,MessageSearchResult->sHubName.Data());
	item->setText(5,spath.Data());
	item->setText(6,MessageSearchResult->sHubHost.Data());

	item->setVisible(visible);

	return res;
}

/** */
void DCHubSearch::GroupSearchResults( eGroupSearchResultType type )
{
	CStringList sl;
	DC_ListResult * item;
	CMessageSearchResult * msg = 0;
	CString s,st;

	ListView_SEARCHRESULT->setUpdatesEnabled(FALSE);
	ListView_SEARCHRESULT->clear();
	ListView_SEARCHRESULT->setRootIsDecorated((type != egsrtNONE));

	while ( (msg=(CMessageSearchResult *)m_pSearchResultList->Next((CObject*)msg)) != 0 )
	{
		item = 0;

		switch(type)
		{
			case egsrtFILE:
				CDir().SplitPathFile(msg->sFile,st,s);
				break;
			case egsrtSIZE:
				s = CString().setNum(msg->lSize);
				break;
			case egsrtNICK:
				s = msg->sNick;
				break;
			case egsrtSLOTS_FREE:
				s = CString().setNum(msg->iFreeSlot);
				break;
			case egsrtHUB:
				s = msg->sHubName;
				break;
			case egsrtPATH:
				CDir().SplitPathFile(msg->sFile,s,st);
				break;
			case egsrtHOST:
				s = msg->sHubHost;
				break;
			case egsrtNONE:
			default:
				s = "";
				break;
		}

		if ( (s != "") && (sl.Get(s,(CObject*&)item) == 0) )
		{
			ShowSearchResult(msg,item);
		}
		else
		{
			item = (DC_ListResult *)ShowSearchResult(msg,ListView_SEARCHRESULT);

			if ( s != "" )
				sl.Add( s, (CObject*)item );
		}
	}

	item = 0;
	s = "";

	while( sl.Next( s, (CObject*&)item ) != 0 )
		sl.Remove(s);
	sl.Clear();

	ListView_SEARCHRESULT->setUpdatesEnabled(TRUE);
	ListView_SEARCHRESULT->triggerUpdate();
}

/** */
void DCHubSearch::DC_LogMessage( CMessageLog * MessageLog )
{
	bool bscroll;

	if ( TextEdit_LOG->verticalScrollBar()->maxValue() == TextEdit_LOG->verticalScrollBar()->value() )
	{
		bscroll = TRUE;
	}
	else
	{
		bscroll = FALSE;
	}

	TextEdit_LOG->append(MessageLog->sMessage.Data());

	if ( bscroll )
	{
		TextEdit_LOG->scrollToBottom();
		TextEdit_LOG->moveCursor( QTextEdit::MoveEnd, FALSE );
	}
}

/** */
CObject * DCHubSearch::GetSearchObject()
{
	CMessageSearchUser * MessageSearchUser = 0;
	CMessageSearchFile * MessageSearchFile = 0;

	if ( LineEdit_SEARCH->text() == "" )
	{
		QMessageBox::critical( this, tr("Hub Search Error"),
			QString(tr("Please enter a searchstring.")));
		return 0;
	}

	// set searchmode
	if ( CheckBox_SEARCHUSER->isChecked() )
	{
		MessageSearchUser = new CMessageSearchUser();
		MessageSearchUser->m_eType = DC_MESSAGE_SEARCH_USER;
		MessageSearchUser->sNick   = LineEdit_SEARCH->text().ascii();

		return MessageSearchUser;
	}

	MessageSearchFile = new CMessageSearchFile();
	MessageSearchFile->m_eType = DC_MESSAGE_SEARCH_FILE;

	MessageSearchFile->sString = LineEdit_SEARCH->text().ascii();

	if ( Config->GetMode() == ecmPassive )
	{
		MessageSearchFile->bLocal = TRUE;

		if ( (RadioButton_CONNECTEDSINGLEHUB->isChecked() == TRUE ) ||
		     (RadioButton_CONNECTEDHUBS->isChecked() == TRUE) )
		{ 
			MessageSearchFile->sSource = Config->GetNick();
		}
		else
		{ 
			MessageSearchFile->sSource = Config->GetSearchNick();
		}
	}
	else
	{
		MessageSearchFile->bLocal  = FALSE;
		MessageSearchFile->sSource = Config->GetUDPHostString();
	}

	if ( MessageSearchFile->sSource == "" )
	{
		delete MessageSearchFile;
		return 0;
	}

	// for exact size searches set size to 0
	if ( ComboBox_SEARCHLIMIT->currentItem() == 2 )
	{
		MessageSearchFile->lSize = 0;
	}
	else
	{
		MessageSearchFile->lSize = SpinBox_SEARCHSIZE->value();
	}

	if ( ComboBox_SEARCHUNIT->currentItem() == 1 )
	{
		MessageSearchFile->lSize *= 1024;
	}
	else if ( ComboBox_SEARCHUNIT->currentItem() == 2 )
	{
		MessageSearchFile->lSize *= 1024*1024;
	}
	else if ( ComboBox_SEARCHUNIT->currentItem() == 3 )
	{
		MessageSearchFile->lSize *= 1024*1024*1024;
	}

	if ( MessageSearchFile->lSize == 0 )
	{
		MessageSearchFile->bSizeLimit = FALSE;
	}

	if ( (ComboBox_SEARCHLIMIT->currentItem() == 0) || (ComboBox_SEARCHLIMIT->currentItem() == 2) || (MessageSearchFile->lSize == 0) )
	{
		MessageSearchFile->bSizeAtMost = FALSE;
	}
	else
	{
		MessageSearchFile->bSizeAtMost = TRUE;
	}

	MessageSearchFile->eFileType = eFileTypes(ComboBox_SEARCHTYPE->currentItem()+1);

	return MessageSearchFile;
}

/** */
void DCHubSearch::slotClickedStop()
{
	StopSearch();

	PushButton_STOP->setEnabled(FALSE);
}

/** */
void DCHubSearch::slotRefreshConnectedHubs()
{
	CString *ps;
	CStringList * StringList;

	ComboBox_CONNECTEDHUBS->clear();

	if ( (StringList = pServerList->GetConnectedHubServerList()) != 0 )
	{
		ps = 0;
		while( StringList->Next((CObject*&)ps) )
		{
			ComboBox_CONNECTEDHUBS->insertItem(ps->Data());
		}

		delete StringList;
	}
}

/** */
void DCHubSearch::slotClickedHistory()
{
	CString *ps;
	CList<CString> list;
	QStringList lst;

	Config->GetSearchHistory( &list );

	if ( list.Count() == 0 )
	{
		return;
	}

	ps = 0;

	while ( (ps=list.Next(ps)) != 0 )
	{
		if ( *ps != "" )
		{
			lst.append(ps->Data());
		}
	}

	bool ok = FALSE;
	QString res = QInputDialog::getItem(
		tr( "Select search" ),
		tr( "Please select an search" ), lst, 1, TRUE, &ok, this );

	if ( ok )
	{
		LineEdit_SEARCH->setText(res);
	}
}

/** */
void DCHubSearch::UpdateHidden( bool SearchFileOnly, int FreeSlots, bool TextFilter )
{
	QListViewItemIterator iter( ListView_SEARCHRESULT );

	ListView_SEARCHRESULT->setUpdatesEnabled(FALSE);

	for ( ; iter.current(); iter++ )
	{
		bool visible = TRUE;
		QString path_file;

		path_file = iter.current()->text(5) + iter.current()->text(0);

		if ( SearchFileOnly )
		{
			// Checks if iter.current() path+filename matches the search pattern
			QString search_pattern;
			for(int j=0; j<=LineEdit_SEARCH->text().contains(" "); j++)
                        {
				search_pattern = LineEdit_SEARCH->text().section(" ",j,j);
				if ( path_file.find(search_pattern,0,FALSE) == -1 )
                                {
					visible = FALSE;
                                }
			}
		}

		// filtering with include
		if (TextFilter && (LineEdit_INCLUDE->text().isEmpty()==FALSE))
		{
			QString search_pattern;
			for(int i=0; search_pattern = LineEdit_INCLUDE->text().section(" ",i,i,QString::SectionSkipEmpty); i++)
			{
				if ( path_file.find(search_pattern,0,FALSE) == -1 )
				{
					visible = FALSE;
				}
			}
		}
		// filtering with exclude
		else if (TextFilter && (LineEdit_EXCLUDE->text().isEmpty()==FALSE))
		{
			QString search_pattern;
			for(int i=0; search_pattern = LineEdit_EXCLUDE->text().section(" ",i,i,QString::SectionSkipEmpty); i++)
			{
				if ( path_file.find(search_pattern,0,FALSE) != -1 )
				{
					visible = FALSE;
				}
			}
		}
		
		if ( visible == TRUE )
		{
			if ( iter.current()->text(3).section("/",0,0).toInt() < FreeSlots )
			{
				visible = FALSE;
			}
		}

		iter.current()->setVisible( visible );
	}

	ListView_SEARCHRESULT->setUpdatesEnabled(TRUE);
	ListView_SEARCHRESULT->triggerUpdate();
}

/** */
void DCHubSearch::slotTextFilterResults()
{
	UpdateHidden( CheckBox_SEARCHFILEONLY->isOn(), SpinBox_FREESLOTS->value(), TRUE );
}

/** */
void DCHubSearch::slotTextFilterReset()
{
	UpdateHidden( CheckBox_SEARCHFILEONLY->isOn(), SpinBox_FREESLOTS->value(), FALSE );
}

/** */
void DCHubSearch::slotToggledSearchFileOnly( bool chkstate )
{
	UpdateHidden( chkstate, SpinBox_FREESLOTS->value() );
}

/** */
void DCHubSearch::slotChangedFreeSlots( int free_slots )
{
	UpdateHidden( CheckBox_SEARCHFILEONLY->isOn(), free_slots );
}

/** */
void DCHubSearch::slotSearchReturnPressed()
{
	CObject * Object;
	eHubSearchHubs hubs;

	// clear messagelist
	SocketCallbackThread.Lock();

	while( (Object = m_pMessageList->Next(0)) != 0 )
		m_pMessageList->Del(Object);

	m_pSearchResultList->Clear();

	SocketCallbackThread.UnLock();

	// clear searchresults & log
	ListView_SEARCHRESULT->clear();
	ListView_SEARCHRESULTUSER->clear();
	TextEdit_LOG->clear();

	// reset counter
	LCDNumber_RESULTS->display(0);

	if ( m_pSearchQueryList )
		delete m_pSearchQueryList;
	m_pSearchQueryList = 0;

	slotAddSearch();

	if ( (m_pSearchQueryList == 0) || (m_pSearchQueryList->Count() == 0) )
	{
		return;
	}

//	Config->AddSearchHistory( LineEdit_SEARCH->text().ascii() );

	if ( Config->GetMode() != ecmPassive )
	{
		if ( Connect( "", Config->GetUDPListenPort(), estUDP ) != 0 )
		{
			QMessageBox::critical( this, tr("Hub Search Error"),
				QString(tr("Connect to local udp port failed. (") + GetSocketError().Data() + ")"));
			return;
		}
	}

	// set searchtype
	if ( RadioButton_CONNECTEDSINGLEHUB->isChecked() )
		hubs = ehshCONNECTEDSINGLE;
	else if ( RadioButton_CONNECTEDHUBS->isChecked() )
		hubs = ehshCONNECTEDALL;
	else if ( RadioButton_AVAILABLEHUBS->isChecked() )
		hubs = ehshPUBLIC;
	else
		hubs = ehshBOOKMARK;

	iMaxThreads = SpinBox_MAXTHREADS->value();

	if ( hubs == ehshCONNECTEDSINGLE )
	{
		CString hubname;

		hubname = ComboBox_CONNECTEDHUBS->currentText().ascii();

		if ( hubname == "" )
		{
			QMessageBox::critical( this, tr("Hub Search Error"),
				QString(tr("Please select a connected hub.")));

			return;
		}
		else if ( StartSearch(hubs,m_pSearchQueryList,hubname) != 0 )
		{
			QMessageBox::critical( this, tr("Hub Search Error"),
				QString(tr("No connected hubs found.")));

			return;
		}
	}
	else if ( hubs == ehshCONNECTEDALL )
	{
		if ( StartSearch(hubs,m_pSearchQueryList) != 0 )
		{
			QMessageBox::critical( this, tr("Hub Search Error"),
				QString(tr("No connected hubs found.")));

			return;
		}
	}
	else
	{
		if ( hubs == ehshPUBLIC )
		{
			if ( StartSearch(hubs,m_pSearchQueryList) != 0 )
			{
				QMessageBox::critical( this, tr("Hub Search Error"),
					QString(tr("No hubs found.")));

				return;
			}
		}
		else
		{
			if ( StartSearch(hubs,m_pSearchQueryList) != 0 )
			{
				QMessageBox::critical( this, tr("Hub Search Error"),
					QString(tr("No hubs found.")));

				return;
			}
		}
	}

	// update view
	SetSearchView( FALSE );

	Timer.start( 500, TRUE );

	return;
}

/** */
void DCHubSearch::slotAddSearch()
{
	CObject * object;

	if ( m_pSearchQueryList == 0 )
		m_pSearchQueryList = new CList<CObject>();

	if ( (object = GetSearchObject()) != 0 )
		m_pSearchQueryList->Add(object);
}

/** */
void DCHubSearch::slotReset()
{
	LineEdit_SEARCH->setText("");
	SpinBox_SEARCHSIZE->setValue(0);
	ComboBox_SEARCHUNIT->setCurrentItem(0);
	ComboBox_SEARCHLIMIT->setCurrentItem(0);
	ComboBox_SEARCHTYPE->setCurrentItem(0);
	SpinBox_FREESLOTS->setValue(0);
}

/** */
void DCHubSearch::slotDoubleClickedSearchResult( QListViewItem * /*item*/ )
{
	QListViewItem * curitem;
	ulonglong size;
	QString localname;
	DCFileTool FileTool;

	curitem   = ListView_SEARCHRESULT->currentItem();
	size      = ((DC_QListViewItem*)curitem)->myvalue;
	localname = curitem->text(0);

	// add file as new source
	FileTool.AddFileSource( curitem->text(2).ascii(),
				curitem->text(4).ascii(),
				curitem->text(6).ascii(),
				(curitem->text(5)+curitem->text(0)).ascii(),
				localname.ascii(),
				"",
				"",
				eltFILE,
				size );

	return;
/*
	int i;
	CString s;
	CByteArray cba;
	QByteArray qba;

	DC_ListResult *item1 = (DC_ListResult*)item;

	QStoredDrag *sd = new QStoredDrag( "dchubsearch/searchresult", this );

	s = item1->text(0).ascii();
	if ( s.Data() != 0 )
		cba.Append( s.Data(), s.Length() );
	cba.Append("\0",1);

	s = s.setNum(item1->myvalue);
	if ( s.Data() != 0 )
		cba.Append( s.Data(), s.Length() );
	cba.Append("\0",1);

	for(i=2;i<=6;i++)
	{
		s = item1->text(i).ascii();
		if ( s.Data() != 0 )
			cba.Append( s.Data(), s.Length() );
		cba.Append("\0",1);
	}

	qba.assign((const char*)cba.Data(),(unsigned int)cba.Size());
	sd->setEncodedData(qba);

	QDragObject *d = sd;
	d->dragCopy();
*/
}

/** */
void DCHubSearch::slotRightButtonClickedSearchResult( QListViewItem * /*item*/, const QPoint &, int column )
{
	DCFileTool filetool;
	CString s;
	int id;
	ulonglong size;
        QPtrList<QListViewItem> selitems;
        QListViewItem * curitem;
	bool isadmin;

	// user have select items ?
	if ( selectedItems( ListView_SEARCHRESULT, selitems ) == 0 )
	{
		return;
	}

	// check if admin for admin-menu
	for ( isadmin = FALSE, curitem = selitems.first(); curitem; curitem = selitems.next() )
	{
		if ( pServerList->IsAdmin(curitem->text(4).ascii()) == FALSE )
		{
			isadmin = FALSE;
			break;
		}
		else
		{
			isadmin = TRUE;
		}
	}
	
	QPopupMenu *m,*msort;

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiDOWNLOAD );
	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_TO, TRUE );
	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_AS, TRUE );
	DCMenuHandler::InsertMenu( m, emiDOWNLOAD_IN, TRUE );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiBROWSE_USER_FILES );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiCOPY_COLUMN_TO_CLIPBOARD );
	DCMenuHandler::InsertMenu( m, emiCOPY_ROW_TO_CLIPBOARD );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiCONNECT_TO_HUB );
	DCMenuHandler::InsertMenu( m, emiADD_BOOKMARK );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );

	// insert submenu
	msort = DCMenuHandler::InsertMenu( m, emisGROUP, TRUE );
	DCMenuHandler::InsertMenu( msort, emiDISABLE_GROUP, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_FILE, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_SIZE, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_NICK, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_SLOTS_FREE, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_HUB, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_PATH, TRUE );
	DCMenuHandler::InsertMenu( msort, emiGROUP_BY_HOST, TRUE );

	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiKICK, isadmin );
	DCMenuHandler::InsertMenu( m, emiFORCE_MOVE, isadmin );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiSAVE, FALSE );
	DCMenuHandler::InsertMenu( m, emiLOAD, FALSE );

	id = m->exec(QCursor::pos());

	delete m;

	if ( (id == emiDOWNLOAD) || (id == emiDOWNLOAD_AS) || (id == emiDOWNLOAD_TO) )
	{
		QString localrootpath = "";
		QString localname = "";

		// select downloadfolder for all selected files
		if ( id == emiDOWNLOAD_TO )
		{
			localrootpath = QFileDialog::getExistingDirectory( "", this, "bdf", tr("Select download folder"), TRUE );

			if ( localrootpath == "" )
				return;
		}

                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			size = ((DC_QListViewItem *)curitem)->myvalue;
			localname = curitem->text(0);

			if ( id == emiDOWNLOAD_AS )
			{
				localrootpath = QFileDialog::getSaveFileName( "", "", this, "bdf", tr("Select file for")+" "+localname );

				if ( localrootpath == "" )
					return;

				QFileInfo fi(localrootpath);
				localrootpath = fi.dirPath();
				localname     = fi.fileName();

				if ( (localrootpath=="") || (localname=="") )
					return;
			}

			// add transfer to the waitlist
			filetool.CheckFile( curitem->text(2).ascii(), curitem->text(4).ascii(), curitem->text(6).ascii(),
						(curitem->text(5)+curitem->text(0)).ascii(), localname.ascii(), "", localrootpath.ascii(), eltFILE,
						size );
		}

		// redraw the list
		ListView_SEARCHRESULT->triggerUpdate();
	}
	else if ( id == emiDOWNLOAD_IN )
	{
		QString localrootpath;
		QString localname;
		QString localpath;

		if ( !(curitem = selitems.first()) )
			return;

		size = ((DC_QListViewItem *)curitem)->myvalue;

		// all files need equal size
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			if ( size != ((DC_QListViewItem *)curitem)->myvalue )
				return;
		}

		if ( filetool.SelectFileSource( size, localname, localrootpath, localpath ) == FALSE )
		{
			return;
		}

                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			// add transfer to the waitlist
			filetool.CheckFile( curitem->text(2).ascii(), curitem->text(4).ascii(), curitem->text(6).ascii(),
					  (curitem->text(5)+curitem->text(0)).ascii(), localname.ascii(), localpath.ascii(), localrootpath.ascii(), eltFILE,
					  size, TRUE);
		}

		// redraw the list
		ListView_SEARCHRESULT->triggerUpdate();
	}
	else if ( id == emiBROWSE_USER_FILES)
	{
		/** add transfer to the waitlist */
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			CString hubname(curitem->text(4).ascii());
			CString hubhost(curitem->text(6).ascii());

			if ( pServerList->IsHubOnline(hubname) == ehsNONE )
			{
				switch( QMessageBox::warning( this, tr("List download"),
							      tr("Not connected to required hub!"),
							      tr("Connect"),
							      tr("Cancel"), 0, 0, 1 ) )
				{
					case 0:
						 pServerList->Connect(hubname.Data(),hubhost.Data());
						 break;
					case 1:
						break;
				}
			}

			pTransferView->DLM_QueueAdd( curitem->text(2).ascii(), curitem->text(4).ascii(), curitem->text(6).ascii(),
						"MyList.DcLst", "MyList.DcLst", "", "", eltBUFFER,
						0, 0 );
		}
	}
	else if ( id == emiCOPY_COLUMN_TO_CLIPBOARD )
	{
		QString s="";
		QClipboard *cb = QApplication::clipboard();
		cb->setText("");

                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			s += curitem->text(column) + "\n";
		}

		cb->setText(s);
	}
	else if ( id == emiCOPY_ROW_TO_CLIPBOARD )
	{
		int idx;
		QString s="";
		QClipboard *cb = QApplication::clipboard();
		cb->setText("");

                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			for( idx = 0; idx < ListView_SEARCHRESULT->columns(); idx++ )
			{
				s += curitem->text(idx) + " ";
			}

			s += "\n";
		}

		cb->setText(s);
	}
	else if ( id == emiCONNECT_TO_HUB )
	{
		// connect to the hub
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			pServerList->Connect( curitem->text(4).ascii(), curitem->text(6).ascii() );
		}
	}
	else if ( id == emiADD_BOOKMARK )
	{
		// bookmark this hub
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			pServerList->AddBookmark( curitem->text(4), curitem->text(6), "" );
		}
	}
	else if ( id == emiDISABLE_GROUP )
	{
		GroupSearchResults( egsrtNONE );
	}
	else if ( id == emiGROUP_BY_FILE )
	{
		GroupSearchResults( egsrtFILE );
	}
	else if ( id == emiGROUP_BY_SIZE )
	{
		GroupSearchResults( egsrtSIZE );
	}
	else if ( id == emiGROUP_BY_NICK )
	{
		GroupSearchResults( egsrtNICK );
	}
	else if ( id == emiGROUP_BY_SLOTS_FREE )
	{
		GroupSearchResults( egsrtSLOTS_FREE );
	}
	else if ( id == emiGROUP_BY_HUB )
	{
		GroupSearchResults( egsrtHUB );
	}
	else if ( id == emiGROUP_BY_PATH )
	{
		GroupSearchResults( egsrtPATH );
	}
	else if ( id == emiGROUP_BY_HOST )
	{
		GroupSearchResults( egsrtHOST );
	}
	else if ( id == emiKICK )
	{
		QString message;

		if ( ! DCClient::GetOPKickMessage( message ) )
		{
			return;
		}

		// kick all user
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			pServerList->OPKick( curitem->text(4), curitem->text(2), message );
		}
	}
	else if ( id == emiFORCE_MOVE )
	{
		QString message, host;

		if ( ! DCClient::GetOPForceMoveMessage( message, host ) )
		{
			return;
		}

		// force move all user
                for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
                {
			pServerList->OPForceMove( curitem->text(4), curitem->text(2), message, host );
		}
	}
}
