/***************************************************************************
                          dcserverlist.cpp  -  description
                             -------------------
    begin                : Mon Oct 1 2001
    copyright            : (C) 2001 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>

#include <qobject.h>
#include <qpainter.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qstringlist.h>
#include <qtabwidget.h>
#include <qinputdialog.h>
#include <qcombobox.h>
#include <qregexp.h>

#include <dcgeturl.h>
#include <dcclient.h>
#include <dcmenuhandler.h>
#include <dcwidget.h>
#include <dchubprofile.h>
#include <dchubfilter.h>

#include "dcserverlist.h"

#include <dclib/dcos.h>
#include <dclib/cbz.h>
#include <dclib/chttp.h>

#include "DCDialogEditServer.h"

#include "res/server.xpm"
#include "res/bookmark_folder.xpm"

DCServerList * pServerList;

/** */
DCServerList::DCServerList(QWidget* parent, const char *name, int wflags) : DCDialogServerList(parent, name, wflags)
{
	int idx;

	// set default icon
	setIcon( QPixmap( server_xpm ) );

	pWorkspace = (QWorkspace*)parent;

	// set the ColumnWidthMode to manual, we will take care of this task
	for( idx = 0; idx < ListView_PUBLIC->columns(); idx++ )
	{
		ListView_PUBLIC->setColumnWidthMode( idx, QListView::Manual );
	}
	for( idx = 0; idx < ListView_BOOKMARKS->columns(); idx++ )
	{
		ListView_BOOKMARKS->setColumnWidthMode( idx, QListView::Manual );
	}

	m_pHubListUrl = 0;
	m_pHttp = new CHttp();
	m_pHttp->SetCallBackFunction( new CCallback<DCServerList>( this, &DCServerList::HttpCallBack ) );

	InitDocument();

	pServerList = this;
}

/** */
DCServerList::~DCServerList()
{
	DCClient * Client = 0;

	while( (Client=(DCClient*)pClientList->Next(0)) != 0 )
	{
		Client->removeEventFilter(this);
		pClientList->Remove(Client);
		Client->Disconnect();
	}

	m_pHttp->SetCallBackFunction(0);
	delete m_pHttp;
}

/** callback function */
int DCServerList::HttpCallBack( CObject * , CObject * object )
{
	CByteArray in,out;
	CBZ bz2;

	switch(((CDCMessage*)object)->m_eType)
	{
		case DC_MESSAGE_CONNECTION_STATE:
		{
			CMessageConnectionState *msg = (CMessageConnectionState*)object;

			if ( msg->eState == estDISCONNECTED )
			{
				if ( m_pHttp->GetData(&in) == TRUE )
				{
					// reset progressbar
					ProgressBar_LOADHUBLIST->setProgress(0);

					if ( m_pHttp->GetUrl().Find(".bz2") != -1 )
					{
						if ( bz2.Decompress( &in, &out ) == TRUE )
						{
							m_HubListData.Append(out.Data(),out.Size());
							m_HubListData.Append("\xD\xA",2);
						}
						else
						{
							printf("bz2 decompress failed\n");
						}
					}
					else
					{
						m_HubListData.Append(in.Data(),in.Size());
						m_HubListData.Append("\xD\xA",2);
					}

				}

				if ( NextPublicHub() == FALSE )
				{
					// printf("ready %ld\n",m_HubListData.Size());
					// show public hub list
				}
			}

			break;
		}

		case DC_MESSAGE_TRANSFER:
		{
			CMessageTransfer *msg = (CMessageTransfer*)object;

			ProgressBar_LOADHUBLIST->setTotalSteps(msg->lLength);
			ProgressBar_LOADHUBLIST->setProgress(msg->lTransfered);

			break;
		}

		default:
			break;
	}

	delete object;

	return 0;
}

/** event filter */
bool DCServerList::eventFilter(QObject* object, QEvent* event)
{
	if((event->type() == QEvent::Close)&&((DCServerList*)object!=this))
	{
		QCloseEvent* e=(QCloseEvent*)event;
		DCClient* pView=(DCClient*)object;

		pView->Disconnect();
		pView->DeInitDocument();

		RemoveHub(pView);

		e->accept();
	}

	return QWidget::eventFilter( object, event );    // standard event processing
}

/** current tab widget change slot */
void DCServerList::slotTabWidgetCurrentChange(QWidget*)
{
	ResizeListViewColumn();
}

/** overridden so that the columns are resized on show() */
void DCServerList::show()
{
	QWidget::show();

	ResizeListViewColumn();
}

/** resize event handler */
void DCServerList::resizeEvent( QResizeEvent * )
{
	ResizeListViewColumn();
}

/** resize the ListView columns */
void DCServerList::ResizeListViewColumn()
{
	int width;

	if ( ListView_PUBLIC->isVisible() )
	{
		width = ListView_PUBLIC->width();

		ListView_PUBLIC->setColumnWidth( 0, ((width*3)/10) );
		ListView_PUBLIC->setColumnWidth( 1, ((width*3)/10) );
		ListView_PUBLIC->setColumnWidth( 2, ((width*3)/10) );
		ListView_PUBLIC->setColumnWidth( 3, ((width*1)/10) );
	}

	if ( ListView_BOOKMARKS->isVisible() )
	{
		width = ListView_BOOKMARKS->width();

		ListView_BOOKMARKS->setColumnWidth( 0, ((width*10)/30) );
		ListView_BOOKMARKS->setColumnWidth( 1, ((width*10)/30) );
		ListView_BOOKMARKS->setColumnWidth( 2, ((width*10)/30) );
	}
}

/** */
int DCServerList::selectedItems( QListView * list, QPtrList<QListViewItem> & lst )
{
	QListViewItemIterator it( (QListView *)list );

	for ( ; it.current(); it++ )
	{
		if ( it.current()->isSelected() )
		{
			lst.append(it.current());
		}
	}

	return lst.count();
}

/** */
void DCServerList::InitDocument()
{
	StringMap * map;
	DC_HubFilterObject * HubFilterObject;

	// hide progressbar
	ProgressBar_LOADHUBLIST->hide();

	// set bookmark icon
	TabWidget_SERVERLIST->setTabIconSet( TabWidget_SERVERLIST->page(1), QIconSet( QPixmap(bookmark_folder_xpm) ) );

//	ListView_PUBLIC->setSorting(-1);
//	ListView_BOOKMARKS->setSorting(-1);
	
	// restore settings
	if ( Config->GetMap("HUBVIEW",map) == TRUE )
	{
		if ( ((*map)["WIDTH"].toInt() > 0) && ((*map)["HEIGTH"].toInt() > 0) )
		{
			setGeometry( (*map)["X"].toInt(), (*map)["Y"].toInt(), (*map)["WIDTH"].toInt(), (*map)["HEIGTH"].toInt() );
		}
	}

	connect( LineEdit_SERVER, SIGNAL(returnPressed()), this, SLOT(slotConnectReturnPressed()) );
	connect( LineEdit_SERVER, SIGNAL(returnPressed()), this, SLOT(slotConnect()) );
	connect( PushButton_RELOADPUBLICHUBLIST, SIGNAL(clicked()), this, SLOT(slotReloadPublicHubList()) );
	connect( PushButton_UPDATEPUBLICHUBLIST, SIGNAL(clicked()), this, SLOT(slotUpdatePublicHubList()) );
	connect( PushButton_CONNECT, SIGNAL(clicked()), this, SLOT(slotConnect()) );

	// public list
	connect( ListView_PUBLIC,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClicked(QListViewItem*)) );
//	connect( ListView_PUBLIC,SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedPublicList(QListViewItem*, const QPoint &, int )) );
	connect( ListView_PUBLIC,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedPublicList(QListViewItem*, const QPoint &, int )) );

	// bookmark list
	connect( ListView_BOOKMARKS,SIGNAL(doubleClicked(QListViewItem*)), this, SLOT(slotDoubleClicked(QListViewItem*)) );
//	connect( ListView_BOOKMARKS,SIGNAL(rightButtonClicked( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedBookmarkList(QListViewItem*, const QPoint &, int )) );
	connect( ListView_BOOKMARKS,SIGNAL(contextMenuRequested( QListViewItem *, const QPoint &, int )), this, SLOT(slotRightButtonClickedBookmarkList(QListViewItem*, const QPoint &, int )) );

	connect( TabWidget_SERVERLIST,SIGNAL(currentChanged(QWidget*)), this, SLOT(slotTabWidgetCurrentChange(QWidget*)) );

	// filter
	connect( PushButton_ADDFILTER, SIGNAL(clicked()), this, SLOT(slotAddFilter()) );
	connect( PushButton_EDITFILTER, SIGNAL(clicked()), this, SLOT(slotEditFilter()) );
	connect( PushButton_DELFILTER, SIGNAL(clicked()), this, SLOT(slotDelFilter()) );
	connect( ComboBox_FILTER, SIGNAL(activated( const QString & )), this, SLOT(slotFilterChange( const QString &)) );

	connect( &m_Timer, SIGNAL(timeout()), this, SLOT(slotGetPublicHubListReady()) );

	// load filter
	Config->LoadDCHubFilter(&m_HubFilterList);

	// create default
	if ( m_HubFilterList.Count() == 0 )
	{
		HubFilterObject = new DC_HubFilterObject();

		HubFilterObject->m_sFilterName = "Default";

		m_HubFilterList.Add( HubFilterObject->m_sFilterName, HubFilterObject );

		Config->SaveDCHubFilter(&m_HubFilterList);
	}

	HubFilterObject = 0;

	while ( m_HubFilterList.Next( (CObject*&)HubFilterObject) == 1 )
	{
		ComboBox_FILTER->insertItem( HubFilterObject->m_sFilterName.Data() );
	}

	ComboBox_FILTER->setCurrentText("Default");

	ShowBookmarkList();
	ShowPublicHubList();
}

/** */
void DCServerList::DeInitDocument()
{
	StringMap * map;

	// save hub view settings
	Config->GetMap("HUBVIEW",map);

	(*map)["X"]         = QString().setNum(x());
	(*map)["Y"]         = QString().setNum(y());
	(*map)["WIDTH"]     = QString().setNum(width());
	(*map)["HEIGTH"]    = QString().setNum(height());
	(*map)["VISIBLE"]   = QString().setNum(isVisible());
	(*map)["MAXIMIZED"] = QString().setNum(isMaximized());
	(*map)["MINIMIZED"] = QString().setNum(isMinimized());
	(*map)["ENABLED"]   = QString().setNum(isEnabled());
}

/** */
void DCServerList::GetPublicHubList()
{
	Config->GetHubListUrlList(&m_HubListUrlList);

	m_pHubListUrl = 0;
	m_HubListData.SetSize(0);

	if ( NextPublicHub() == TRUE )
	{
		// disable buttons
		PushButton_RELOADPUBLICHUBLIST->setEnabled(FALSE);
		PushButton_UPDATEPUBLICHUBLIST->setEnabled(FALSE);

		// reset progressbar
		ProgressBar_LOADHUBLIST->setProgress(0);
		// show progressbar
		ProgressBar_LOADHUBLIST->show();

		// start poll timer
		m_Timer.start( 500, TRUE );
	}
	else
	{
		// hide progressbar
		ProgressBar_LOADHUBLIST->hide();
	}
}

/** */
bool DCServerList::NextPublicHub()
{
	bool res = FALSE;

	while( (m_pHubListUrl=m_HubListUrlList.Next(m_pHubListUrl)) != 0 )
	{
		if ( m_pHubListUrl->bEnabled == TRUE )
		{
			m_pHttp->GetUrl(m_pHubListUrl->sUrl);
			res = TRUE;
			break;
		}
	}

	return res;
}

/** */
void DCServerList::ParsePublicHubList( CByteArray * ba )
{
	QString line;
	CString s;
	QString s1,s2,s3,s4;
	int i=0,i1=0;

	if ( ba->Size() == 0 )
	{
		return;
	}

	s.Set((const char*)ba->Data(),ba->Size());

	while( (i = s.Find(0x0d,i)) != -1 )
	{
		line = s.Mid(i1,i-i1).Data();

		if ( !line.isEmpty() )
		{
			s1 = line.section( "|", 0, 0 );
			s2 = line.section( "|", 1, 1 );
			s3 = line.section( "|", 2, 2 );
			s4 = line.section( "|", 3, 3 );

			if ( s1.isEmpty() ) s1 = QString::null;
			if ( s2.isEmpty() ) s2 = QString::null;
			if ( s3.isEmpty() ) s3 = QString::null;
			if ( s4.isEmpty() ) s4 = QString::null;

			// replace all spaces
			s2 = s2.replace(QRegExp(" "),"");

			Config->AddPublicHub( s1.ascii(), s2.ascii(), s3.ascii(), s4.ascii() );
		}

		i1 = i+2;
		i += 2;
	}

	ShowPublicHubList();

	// store the list
	if ( Config->GetHubListStoreLocal() == TRUE )
		Config->SaveDCHub();
}

/** */
void DCServerList::ShowPublicHubList()
{
	DCConfigHubItem * hubitem;
	CList<DCConfigHubItem> PublicHubList;
	unsigned int VisibleHubs,minuser;
	DC_HubFilterObject * HubFilterObject = 0;
	CString s;

	LineEdit_SERVERCOUNT->setText("");

	ListView_PUBLIC->setUpdatesEnabled(FALSE);
	ListView_PUBLIC->clear();

	Config->GetPublicHubList(&PublicHubList);

	VisibleHubs = 0;
	hubitem     = 0;
	minuser     = 0;

	// get the hubfilter
	s = ComboBox_FILTER->currentText().ascii();

	if ( s != "" )
	{
		m_HubFilterList.Get( s, (CObject *&)HubFilterObject );
	}

	while( (hubitem=PublicHubList.Next(hubitem)) != 0 )
	{
		DC_QListViewItem *item = new DC_QListViewItem(ListView_PUBLIC);
		item->myvalue = hubitem->sUserCount.asULL();
		item->mycol = 3;

		item->setText(0,hubitem->sName.Data());
		item->setText(1,hubitem->sHost.Data());
		item->setText(2,hubitem->sDescription.Data());
		item->setText(3,hubitem->sUserCount.Data());

		// use the filter
		if ( HubFilterObject )
		{
			if ( item->myvalue < HubFilterObject->m_nUser )
			{
				item->setVisible(FALSE);
			}
			else
			{
				QString AllFields = 0;
				QString Pattern;
				QString Contains;
				int i=0;

				if ( HubFilterObject->m_bName )
					AllFields += item->text(0);
				if ( HubFilterObject->m_bServer )
					AllFields += item->text(1);
				if ( HubFilterObject->m_bDescription )
					AllFields += item->text(2);

				if ( AllFields != "" )
				{
					Contains = HubFilterObject->m_sContains.Data();

					while ( !(Pattern=Contains.section(" ",i,i,QString::SectionSkipEmpty)).isEmpty())
					{
						i++;
						if ( AllFields.contains(Pattern,FALSE) == 0 )
						{
							item->setVisible(FALSE);
							break;
						}
					}
				}
			}
		}

		if ( item->isVisible() )
		{
			VisibleHubs++;
		}
	}

	ListView_PUBLIC->setUpdatesEnabled(TRUE);
	ListView_PUBLIC->triggerUpdate();

	LineEdit_SERVERCOUNT->setText(QString().setNum(VisibleHubs)+"/"+QString().setNum(ListView_PUBLIC->childCount()));
}

/** */
void DCServerList::ShowBookmarkList()
{
	DCConfigHubItem * hubitem = 0;
	CList<DCConfigHubItem> list;
	DCConfigHubProfile ConfigHubProfile;

	Config->GetBookmarkHubList(&list);

	ListView_BOOKMARKS->setUpdatesEnabled(FALSE);
	ListView_BOOKMARKS->clear();

	while( (hubitem=list.Next(hubitem)) != 0 )
	{
		new QListViewItem(ListView_BOOKMARKS,hubitem->sName.Data(),hubitem->sHost.Data(),hubitem->sDescription.Data());
	}

	ListView_BOOKMARKS->setUpdatesEnabled(TRUE);
	ListView_BOOKMARKS->triggerUpdate();

	LineEdit_BOOKMARKCOUNT->setText(QString().setNum(ListView_BOOKMARKS->childCount()));
}

/** */
void DCServerList::AutoConnect()
{
	DCConfigHubItem * hubitem = 0;
	CList<DCConfigHubItem> list;
	DCConfigHubProfile ConfigHubProfile;

	Config->GetBookmarkHubList(&list);

	while( (hubitem=list.Next(hubitem)) != 0 )
	{
		// check for autoconnect
		if ( Config->GetHubProfile( hubitem->sProfile, &ConfigHubProfile ) == TRUE )
		{
			if ( ConfigHubProfile.bAutoConnect == TRUE )
			{
				Connect( hubitem->sName.Data(), hubitem->sHost.Data() );
			}
		}
	}
}

/** */
void DCServerList::slotGetPublicHubListReady()
{
	if ( m_pHubListUrl == 0 )
	{
		ParsePublicHubList(&m_HubListData);

		// enable buttons
		PushButton_RELOADPUBLICHUBLIST->setEnabled(TRUE);
		PushButton_UPDATEPUBLICHUBLIST->setEnabled(TRUE);

		// hide progressbar
		ProgressBar_LOADHUBLIST->hide();
	}
	else
	{
		m_Timer.start( 500, TRUE );
	}
}

/** */
void DCServerList::slotReloadPublicHubList()
{
	Config->ClearPublicHubList();

	GetPublicHubList();
}

/** update public hub list */
void DCServerList::slotUpdatePublicHubList()
{
	GetPublicHubList();
}

/** */
void DCServerList::slotDoubleClicked( QListViewItem * item )
{
	if ( item )
		Connect( item->text(0).ascii(), item->text(1).ascii() );
}

/** */
void DCServerList::slotConnect()
{
	Connect( "", LineEdit_SERVER->text().ascii() );
}

/** */
void DCServerList::slotConnectReturnPressed()
{
	Connect( "", LineEdit_SERVER->text().ascii() );
}

/** */
void DCServerList::slotAddFilter()
{
	CString s;
	DC_HubFilterObject * HubFilterObject, * HubFilterObject1 ;
	DCHubFilter * hf;

	hf = new DCHubFilter(this);

	if ( hf->exec() == QDialog::Accepted )
	{
		HubFilterObject = new  DC_HubFilterObject();

		// get the new data
		hf->Get( HubFilterObject );

		if ( HubFilterObject->m_sFilterName == "" )
		{
			// todo: error message

			delete HubFilterObject;
		}
		else if ( m_HubFilterList.Get( HubFilterObject->m_sFilterName, (CObject *&)HubFilterObject1 ) == 0 )
		{
			// todo: error message allready exist

			delete HubFilterObject;
		}
		else
		{
			m_HubFilterList.Add( HubFilterObject->m_sFilterName, HubFilterObject );

			ComboBox_FILTER->insertItem( HubFilterObject->m_sFilterName.Data() );
		}

		Config->SaveDCHubFilter(&m_HubFilterList);

		delete hf;
	}
}

/** */
void DCServerList::slotEditFilter()
{
	CString s;
	DC_HubFilterObject * HubFilterObject, * HubFilterObject1;
	DCHubFilter * hf;

	s = ComboBox_FILTER->currentText().ascii();

	if ( (s != "") && (s != "Default") )
	{
		if ( m_HubFilterList.Get( s, (CObject *&)HubFilterObject ) == 0 )
		{
			hf = new DCHubFilter(this);

			hf->Edit( HubFilterObject );

			if ( hf->exec() == QDialog::Accepted )
			{
				// get the new data
				hf->Get( HubFilterObject );

				if ( HubFilterObject->m_sFilterName == "" )
				{
					HubFilterObject->m_sFilterName = s;

					// todo: error message
				}
				else if ( s != HubFilterObject->m_sFilterName )
				{
					if ( m_HubFilterList.Get( HubFilterObject->m_sFilterName, (CObject *&)HubFilterObject1 ) == 0 )
					{
						// todo: error message allready exist
					}
					else
					{
						m_HubFilterList.Remove(s);
						m_HubFilterList.Add( HubFilterObject->m_sFilterName, HubFilterObject );

						ComboBox_FILTER->changeItem( HubFilterObject->m_sFilterName.Data(), ComboBox_FILTER->currentItem() );
					}
				}

				// update view
				slotFilterChange( HubFilterObject->m_sFilterName.Data() );

				Config->SaveDCHubFilter(&m_HubFilterList);
			}

			delete hf;
		}
	}
}

/** */
void DCServerList::slotDelFilter()
{
	CString s;

	s = ComboBox_FILTER->currentText().ascii();

	if ( (s != "") && (s != "Default") )
	{
		ComboBox_FILTER->removeItem(ComboBox_FILTER->currentItem());

		m_HubFilterList.Del(s);

		Config->SaveDCHubFilter(&m_HubFilterList);
	}
}

/** */
void DCServerList::slotFilterChange( const QString & s )
{
	if ( s != "" )
	{
		ShowPublicHubList();
	}
}

/** */
void DCServerList::Connect( CString hubname, CString server )
{
	DCClient * client;

	if ( server == "" )
	{
		return;
	}

	if ( hubname == "" )
	{
		hubname = server;
	}

	m_Thread.Lock();

	client = (DCClient*)GetHub(hubname);

	if ( client )
	{
		client->setFocus();
	}

	m_Thread.UnLock();

	if ( client )
	{
		return;
	}

	client = new DCClient(pWorkspace,0,WDestructiveClose);

	client->setCaption(hubname.Data());

	client->installEventFilter(this);

	if ( !Config->GetOpenClientWindowsMinimized() )
	{
		if ( pWorkspace->windowList().isEmpty() ) // show the very first window in maximized mode
			client->showMaximized();
		else
			client->show();
	}
	else
	{
		client->showMinimized();
	}

	CServerManager::Connect(hubname,server,client);
}

/** */
void DCServerList::DelBookmark( QString name, QString host, QString description )
{
	if ( Config->RemoveBookmarkHub( name.ascii(), host.ascii(), description.ascii() ) == TRUE )
	{
		Config->SaveDCBookHub();
	}
}

/** */
void DCServerList::AddBookmark( QString name, QString host, QString description )
{
	if ( Config->AddBookmarkHub( name.ascii(), host.ascii(), description.ascii() ) == TRUE )
	{
		new QListViewItem( ListView_BOOKMARKS, name, host, description );

		Config->SaveDCBookHub();
	}
}

/** */
void DCServerList::UpdateBookmark( QString name, QString host, QString description )
{
	if ( Config->UpdateBookmarkHub( name.ascii(), host.ascii(), description.ascii() ) == TRUE )
	{
		Config->SaveDCBookHub();
	}
}

/** */
void DCServerList::OpenPrivateChat( QString hubname, QString nick )
{
        DCClient * Client = 0;

	m_Thread.Lock();

	Client = (DCClient*)GetHub(hubname.ascii());

	if ( Client )
	{
		Client->DC_PrivateChat(nick,"","",TRUE);
	}

	m_Thread.UnLock();
}

/** */
void DCServerList::CloseAllChats()
{
	CClient * Client = 0;

	m_Thread.Lock();

	while( (Client=pClientList->Next(Client)) != 0 )
	{
		((DCClient*)Client)->CloseAllChats();
	}

	m_Thread.UnLock();
}

/** */
void DCServerList::OPKick( QString hubname, QString nick, QString message )
{
        DCClient * Client = 0;

	m_Thread.Lock();

	Client = (DCClient*)GetHub(hubname.ascii());

	if ( Client )
	{
		Client->OPKick( nick, message );
	}

	m_Thread.UnLock();
}

/** */
void DCServerList::OPForceMove( QString hubname, QString nick, QString message, QString host )
{
        DCClient * Client = 0;

	m_Thread.Lock();

	Client = (DCClient*)GetHub(hubname.ascii());

	if ( Client )
	{
		Client->OPForceMove( nick, message, host );
	}

	m_Thread.UnLock();
}

/** */
void DCServerList::slotRightButtonClickedBookmarkList( QListViewItem * /*item*/, const QPoint &, int )
{
	int id;
	QPopupMenu *m;
	DCDialogEditServer * des;
	QPtrList<QListViewItem> selitems;
	QListViewItem * curitem;

	selectedItems(ListView_BOOKMARKS,selitems);

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiADD );
	DCMenuHandler::InsertMenu( m, emiEDIT, (selitems.count() == 1) );
	DCMenuHandler::InsertMenu( m, emiREMOVE, (selitems.count() > 0) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiCONNECT, (selitems.count() > 0) );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiUPDATE_SERVER, (selitems.count() > 0) );
	DCMenuHandler::InsertMenu( m, emiUPDATE_ALL_SERVER );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiEDIT_HUB_PROFILES );
	DCMenuHandler::InsertMenu( m, emiSET_HUB_PROFILE, (selitems.count() > 0) );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == -1 )
	{
		return;
	}

	if ( id == emiADD )
	{
		des = new DCDialogEditServer();
		des->setCaption(tr("Add Bookmark"));

		if ( des->exec() == QDialog::Accepted )
		{
			AddBookmark( des->LineEdit_NAME->text(), des->LineEdit_HOST->text(), des->LineEdit_DESCRIPTION->text() );
			LineEdit_BOOKMARKCOUNT->setText(QString().setNum(ListView_BOOKMARKS->childCount()));
		}

		delete des;
	}
	else if ( id == emiEDIT )
	{
		// edit only for one selected item
		curitem = selitems.first();

		des = new DCDialogEditServer();
		des->setCaption(tr("Edit Bookmark"));

		des->LineEdit_NAME->setText(curitem->text(0));
		des->LineEdit_HOST->setText(curitem->text(1));
		des->LineEdit_DESCRIPTION->setText(curitem->text(2));

		if ( des->exec() == QDialog::Accepted  )
		{
			// quick & dirty ...
			DelBookmark( curitem->text(0), curitem->text(1), curitem->text(2) );
			ListView_BOOKMARKS->takeItem(curitem);
			delete curitem;
			AddBookmark( des->LineEdit_NAME->text(), des->LineEdit_HOST->text(), des->LineEdit_DESCRIPTION->text() );
			LineEdit_BOOKMARKCOUNT->setText(QString().setNum(ListView_BOOKMARKS->childCount()));
		}

		delete des;
	}
	else if ( id == emiREMOVE )
	{
	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			DelBookmark( curitem->text(0), curitem->text(1), curitem->text(2) );
			ListView_BOOKMARKS->takeItem(curitem);
		}

		LineEdit_BOOKMARKCOUNT->setText(QString().setNum(ListView_BOOKMARKS->childCount()));
	}
	else if ( id == emiCONNECT )
	{
	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			Connect( curitem->text(0).ascii(), curitem->text(1).ascii() );
		}
	}
	else if ( id == emiUPDATE_SERVER )
	{
		DCConfigHubItem hubitem;

	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			if ( Config->GetPublicHub( curitem->text(0).ascii(), &hubitem ) == TRUE )
			{
				UpdateBookmark( hubitem.sName.Data(), hubitem.sHost.Data(), hubitem.sDescription.Data() );
				curitem->setText(1,hubitem.sHost.Data());
				curitem->setText(2,hubitem.sDescription.Data());
			}
		}
	}
	else if ( id == emiUPDATE_ALL_SERVER )
	{
		DCConfigHubItem * hubitem = 0;
		DCConfigHubItem hubitem1;
		CList<DCConfigHubItem> list;

		Config->GetBookmarkHubList(&list);

		while( (hubitem=list.Next(hubitem)) != 0 )
		{
			if ( Config->GetPublicHub( hubitem->sName, &hubitem1 ) == TRUE )
			{
				Config->UpdateBookmarkHub( hubitem1.sName, hubitem1.sHost, hubitem1.sDescription );
			}
		}

		Config->SaveDCBookHub();

		ShowBookmarkList();
	}
	else if ( id == emiEDIT_HUB_PROFILES )
	{
		// edit all hub profiles
		DCHubProfile * d = new DCHubProfile();

		d->exec();

		delete d;
	}
	else if ( id == emiSET_HUB_PROFILE )
	{
		// set hub profile for the selected bookmarks
		QStringList lst;
		CStringList list;
		DCConfigHubProfile * pConfigHubProfile = 0;

	 	Config->GetHubProfileList( &list );

		while ( list.Next((CObject*&)pConfigHubProfile) )
		{
			if ( pConfigHubProfile->sName != "" )
				lst.append(pConfigHubProfile->sName.Data());
		}

		bool ok = FALSE;
		QString res = QInputDialog::getItem(
			tr( "Select profle" ),
			tr( "Please select an profile" ), lst, 1, TRUE, &ok, this );

		if ( ok )
		{
		        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
		        {
				Config->SetBookmarkHubProfile(curitem->text(0).ascii(),res.ascii());
			}

			Config->SaveDCBookHub();
		}
	}
}

/** */
void DCServerList::slotRightButtonClickedPublicList( QListViewItem * /*item*/, const QPoint &, int )
{
	int id;
	QPopupMenu *m;
	QPtrList<QListViewItem> selitems;
	QListViewItem * curitem;

	if ( selectedItems(ListView_PUBLIC,selitems) == 0 )
	{
		return;
	}

	m = new QPopupMenu(this);

	DCMenuHandler::InsertMenu( m, emiCONNECT );
	DCMenuHandler::InsertMenu( m, emiSEPARATOR );
	DCMenuHandler::InsertMenu( m, emiADD_BOOKMARK );

	id = m->exec(QCursor::pos());

	delete m;

	if ( id == -1 )
	{
		return;
	}

	if ( id == emiCONNECT )
	{
	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			Connect( curitem->text(0).ascii(), curitem->text(1).ascii() );
		}
	}
	else if ( id == emiADD_BOOKMARK )
	{
	        for ( curitem = selitems.first(); curitem; curitem = selitems.next() )
	        {
			AddBookmark( curitem->text(0), curitem->text(1), curitem->text(2) );
		}

		LineEdit_BOOKMARKCOUNT->setText(QString().setNum(ListView_BOOKMARKS->childCount()));
	}
}
