/***************************************************************************
                          dcserverlist.h  -  description
                             -------------------
    begin                : Mon Oct 1 2001
    copyright            : (C) 2001 by Mathias Kster
    email                : mathen@ketelhot.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DCSERVERLIST_H
#define DCSERVERLIST_H

/**
  *@author Mathias Kster
  */

#include <qtimer.h>

#include <DCDialogServerList.h>

#include <dcconfig.h>

#include <dclib/cbytearray.h>
#include <dclib/cstringlist.h>
#include <dclib/cservermanager.h>

class DCClient;
class QStringList;
class QListView;
class QWorkspace;
class CHttp;

class DCServerList : public DCDialogServerList, public CServerManager {
Q_OBJECT
public:
	/** */
	DCServerList(QWidget* parent, const char *name, int wflags);
	/** */
	virtual ~DCServerList();

	/** */
	void InitDocument();
	/** */
	void DeInitDocument();
	/** */
	void GetPublicHubList();
	/** */
	bool NextPublicHub();
	/** */
	void ShowPublicHubList();
	/** */
	void ShowBookmarkList();
	/** */
	void ParsePublicHubList( CByteArray * ba );
	/** */
	void AutoConnect();
	/** */
	void Connect( CString hubname, CString server );
	/** */
	void AddBookmark( QString name, QString host, QString description );
	/** */
	void DelBookmark( QString name, QString host, QString description );
	/** */
	void UpdateBookmark( QString name, QString host, QString description );

	/** */
	void OPKick( QString hubname, QString nick, QString message );
	/** */
	void OPForceMove( QString hubname, QString nick, QString message, QString host );

	/** */
	void OpenPrivateChat( QString hubname, QString nick );
	/** */
	void CloseAllChats();

	/** overridden so that the columns are resized on show() */
	virtual void show();

protected:
	/** */
	QWorkspace *pWorkspace;

	/** resize the ListView columns */
	void ResizeListViewColumn();

	/** event filter */
	virtual bool eventFilter(QObject* object, QEvent* event);
	/** close event handler */
	virtual void closeEvent(QCloseEvent*) {};
	/** resize event handler */
	virtual void resizeEvent( QResizeEvent * );

private:
	/** */
	int selectedItems( QListView * list, QPtrList<QListViewItem> & lst );
	/** callback function */
	int HttpCallBack( CObject *, CObject * );

	/** */
	CStringList m_HubFilterList;
	/** */
	CHttp * m_pHttp;
	/** */
	CList<DCConfigHubListUrl> m_HubListUrlList;
	/** */
	DCConfigHubListUrl * m_pHubListUrl;
	/** */
	CByteArray m_HubListData;
	/** */
	QTimer m_Timer;

private slots:
	/** reload public hub list */
	void slotReloadPublicHubList();
	/** update public hub list */
	void slotUpdatePublicHubList();
	/** */
	void slotDoubleClicked(QListViewItem*item);
	/** */
	void slotConnect();
	/** */
	void slotConnectReturnPressed();
	/** */
	void slotRightButtonClickedPublicList(QListViewItem*, const QPoint &, int );
	/** */
	void slotRightButtonClickedBookmarkList(QListViewItem*, const QPoint &, int );
	/** current tab widget change slot */
	void slotTabWidgetCurrentChange(QWidget*);
	/** */
	void slotAddFilter();
	/** */
	void slotEditFilter();
	/** */
	void slotDelFilter();
	/** */
	void slotFilterChange( const QString &);
	/** */
	void slotGetPublicHubListReady();
};

/** */
extern DCServerList * pServerList;

#endif
